@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -82,6 +82,7 @@ protected:
   // If needed, to be used by a Multicommand that changes the DB or
   // a single command that changes the DB outside a MultiCommand
   std::vector<StringX> m_vSavedModifiedNodes;
+  std::vector<StringX> m_vSavedModifiedEmptyGroups;
 
   // This is the potential change type if it there is anything to do
   // It is set so that MultiCommands can tell whether to save DB information
@@ -169,20 +170,20 @@ public:
   enum Function {NP_ADDNEW = 0, NP_REPLACEALL};
 
   static DBPolicyNamesCommand *Create(CommandInterface *pcomInt,
-                                PSWDPolicyMap &MapPSWDPLC,
+                                const PSWDPolicyMap &MapPSWDPLC,
                                 Function function)
   { return new DBPolicyNamesCommand(pcomInt, MapPSWDPLC, function); }
   static DBPolicyNamesCommand *Create(CommandInterface *pcomInt,
-                                StringX &sxPolicyName, PWPolicy &st_pp)
+                                const StringX &sxPolicyName, const PWPolicy &st_pp)
   { return new DBPolicyNamesCommand(pcomInt, sxPolicyName, st_pp); }
   int Execute();
   void Undo();
 
 private:
-  DBPolicyNamesCommand(CommandInterface *pcomInt, PSWDPolicyMap &MapPSWDPLC,
+  DBPolicyNamesCommand(CommandInterface *pcomInt, const PSWDPolicyMap &MapPSWDPLC,
                        Function function);
-  DBPolicyNamesCommand(CommandInterface *pcomInt, StringX &sxPolicyName,
-                       PWPolicy &st_pp);
+  DBPolicyNamesCommand(CommandInterface *pcomInt, const StringX &sxPolicyName,
+                       const PWPolicy &st_pp);
 
   PSWDPolicyMap m_OldMapPSWDPLC;
   PSWDPolicyMap m_NewMapPSWDPLC;
