@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -73,7 +73,8 @@ class CommandInterface {
   virtual void UndoChangeHeader(const StringX &sxOldValue, const PWSfile::HeaderType ht) = 0;
   virtual StringX GetHeaderItem(PWSfile::HeaderType ht) = 0;
 
-  virtual void AddChangedNodes(StringX path) = 0;
+  virtual void AddChangedNodes(const StringX &path) = 0;
+  virtual void AddChangedEmptyGroups(const StringX &path) = 0;
   
   virtual const CItemData *GetBaseEntry(const CItemData *pAliasOrSC) const = 0;
   virtual const ItemMMap &GetBase2AliasesMmap() const = 0;
@@ -118,12 +119,17 @@ class CommandInterface {
   void SetModifiedNodes(const std::vector<StringX> &saved_vNodes_Modified)
   { m_vModifiedNodes = saved_vNodes_Modified; }
 
-  
+  std::vector<StringX> &GetModifiedEmptyGroups() { return m_vModifiedEmptyGroups; }
+  void SetModifiedEmptyGroups(const std::vector<StringX> &saved_vEmptyGroups_Modified)
+  { m_vModifiedEmptyGroups = saved_vEmptyGroups_Modified; }
+    
   virtual ~CommandInterface() {}
 
  protected:
   // Changed groups
   std::vector<StringX> m_vModifiedNodes;
+  // Changed empty groups
+  std::vector<StringX> m_vModifiedEmptyGroups;
 };
 
 #endif /* __COMMANDINTERFACE_H */
