@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -132,6 +132,9 @@ public:
   bool HasUUID() const                     { return IsFieldSet(ATTUUID);   }
   bool IsTitleSet() const                  { return IsFieldSet(ATTTITLE);     }
   bool IsCreationTimeSet() const           { return IsFieldSet(ATTCTIME);     }
+    
+  void SerializePlainText(std::vector<char> &v) const;
+  bool DeSerializePlainText(const std::vector<char> &v);
 
 private:
   bool SetField(unsigned char type, const unsigned char *data, size_t len);
