@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -339,6 +339,8 @@ void PWSMatch::GetMatchType(MatchType mtype,
       // Note: purpose drop through to standard 'string' processing
       //[[fallthrough]];
     case MT_STRING:
+      //[[fallthrough]];
+    case MT_MEDIATYPE:
       cs1 = fstring;
       LoadAString(cs2, fcase ? IDSC_CASE_SENSITIVE : IDSC_CASE_INSENSITIVE);
       break;
@@ -396,6 +398,8 @@ void PWSMatch::GetMatchType(MatchType mtype,
     case MT_DCA:
     case MT_SHIFTDCA:
       switch (fdca) {
+        case -2: // Default value if present or not present
+          return;
         case -1:                                        id = IDSC_CURRENTDEFAULTDCA;  break;
         case PWSprefs::DoubleClickCopyPassword:         id = IDSC_DCACOPYPASSWORD;    break;
         case PWSprefs::DoubleClickViewEdit:             id = IDSC_DCAVIEWEDIT;        break;
@@ -455,9 +459,6 @@ void PWSMatch::GetMatchType(MatchType mtype,
           Format(cs2, L"%d", fnum2 >> (funit * 10));
       }
       break;
-    case MT_MEDIATYPE:
-      cs1 = fstring;
-      break;
     default:
       ASSERT(0);
   }
