@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -137,7 +137,7 @@ int PWSfileV3::Close()
     // We're here *after* TERMINAL_BLOCK has been read
     // and detected (by _readcbc) - just read hmac & verify
     unsigned char d[SHA256::HASHLEN];
-    fread(d, sizeof(d), 1, m_fd);
+    (void) fread(d, sizeof(d), 1, m_fd);
     if (memcmp(d, digest, SHA256::HASHLEN) == 0)
       return PWSfile::Close();
     else {
@@ -222,10 +222,10 @@ int PWSfileV3::CheckPasskey(const StringX &filename,
 
   fseek(fd, sizeof(V3TAG), SEEK_SET); // skip over tag
   unsigned char salt[PWSaltLength];
-  fread(salt, 1, sizeof(salt), fd);
+  (void) fread(salt, 1, sizeof(salt), fd);
 
   unsigned char Nb[sizeof(uint32)];
-  fread(Nb, 1, sizeof(Nb), fd);
+  (void) fread(Nb, 1, sizeof(Nb), fd);
   { // block to shut up compiler warning w.r.t. goto
     const uint32 N = getInt32(Nb);
 
@@ -244,7 +244,7 @@ int PWSfileV3::CheckPasskey(const StringX &filename,
   H.Update(usedPtag, SHA256::HASHLEN);
   H.Final(HPtag);
   unsigned char readHPtag[SHA256::HASHLEN];
-  fread(readHPtag, 1, sizeof(readHPtag), fd);
+  (void) fread(readHPtag, 1, sizeof(readHPtag), fd);
   if (memcmp(readHPtag, HPtag, sizeof(readHPtag)) != 0) {
     retval = WRONG_PASSWORD;
     goto err;
@@ -603,7 +603,7 @@ int PWSfileV3::ReadHeader()
 
   unsigned char B1B2[sizeof(m_key)];
   ASSERT(sizeof(B1B2) == 32); // Generalize later
-  fread(B1B2, 1, sizeof(B1B2), m_fd);
+  (void) fread(B1B2, 1, sizeof(B1B2), m_fd);
   TwoFish TF(Ptag, sizeof(Ptag));
   TF.Decrypt(B1B2, m_key);
   TF.Decrypt(B1B2 + 16, m_key + 16);
@@ -611,13 +611,13 @@ int PWSfileV3::ReadHeader()
   unsigned char L[32]; // for HMAC
   unsigned char B3B4[sizeof(L)];
   ASSERT(sizeof(B3B4) == 32); // Generalize later
-  fread(B3B4, 1, sizeof(B3B4), m_fd);
+  (void) fread(B3B4, 1, sizeof(B3B4), m_fd);
   TF.Decrypt(B3B4, L);
   TF.Decrypt(B3B4 + 16, L + 16);
 
   m_hmac.Init(L, sizeof(L));
 
-  fread(m_ipthing, 1, sizeof(m_ipthing), m_fd);
+  (void) fread(m_ipthing, 1, sizeof(m_ipthing), m_fd);
 
   m_fish = new TwoFish(m_key, sizeof(m_key));
 
@@ -773,52 +773,53 @@ int PWSfileV3::ReadHeader()
         m_hdr.m_DB_Description = text;
         break;
 
+      case HDR_FILTERS:
+        if (utf8 != nullptr) utf8[utf8Len] = '\0';
+        if (utf8Len == 0) break;
+        utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
+        if (utf8Len > 0) {
+          stringT strErrors;
 #if !defined(USE_XML_LIBRARY) || (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
-        // Don't support importing XML from non-Windows platforms
-        // using Microsoft XML libraries
-        // Will be treated as an 'unknown header field' by the 'default' clause below
+          // Using PUGI XML we do not need XDS File
+          stringT XSDFilename = _T("");
 #else
-    case HDR_FILTERS:
-      if (utf8 != nullptr) utf8[utf8Len] = '\0';
-      utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
-      if (utf8Len > 0) {
-        stringT strErrors;
-        stringT XSDFilename = PWSdirs::GetXMLDir() + _T("pwsafe_filter.xsd");
-        if (!pws_os::FileExists(XSDFilename)) {
-          // No filter schema => user won't be able to access stored filters
-          // Inform her of the fact (probably an installation problem).
-          stringT message, message2;
-          Format(message, IDSC_MISSINGXSD, L"pwsafe_filter.xsd");
-          LoadAString(message2, IDSC_FILTERSKEPT);
-          message += stringT(_T("\n\n")) + message2;
-          if (m_pReporter != nullptr)
-            (*m_pReporter)(message);
+          stringT XSDFilename = PWSdirs::GetXMLDir() + _T("pwsafe_filter.xsd");
+          if (!pws_os::FileExists(XSDFilename)) {
+            // No filter schema => user won't be able to access stored filters
+            // Inform her of the fact (probably an installation problem).
+            stringT message, message2;
+            Format(message, IDSC_MISSINGXSD, L"pwsafe_filter.xsd");
+            LoadAString(message2, IDSC_FILTERSKEPT);
+            message += stringT(_T("\n\n")) + message2;
+            if (m_pReporter != nullptr)
+              (*m_pReporter)(message);
 
-          // Treat it as an Unknown field!
-          // Maybe user used a later version of PWS
-          // and we don't want to lose anything
-          UnknownFieldEntry unkhfe(fieldType, utf8Len, utf8);
-          m_UHFL.push_back(unkhfe);
-          break;
-        }
-        int rc = m_MapDBFilters.ImportFilterXMLFile(FPOOL_DATABASE, text.c_str(), _T(""),
-                                                    XSDFilename.c_str(),
-                                                    strErrors, m_pAsker);
-        if (rc != PWScore::SUCCESS) {
-          // Can't parse it - treat as an unknown field,
-          // Notify user that filter won't be available
-          stringT message;
-          LoadAString(message, IDSC_CANTPROCESSDBFILTERS);
-          if (m_pReporter != nullptr)
-            (*m_pReporter)(message);
-          pws_os::Trace(L"Error while parsing header filters.\n\tData: %ls\n\tErrors: %ls\n",
-                        text.c_str(), strErrors.c_str());
-          UnknownFieldEntry unkhfe(fieldType, utf8Len, utf8);
-          m_UHFL.push_back(unkhfe);
-        }
-      }
-      break;
+            // Treat it as an Unknown field!
+            // Maybe user used a later version of PWS
+            // and we don't want to lose anything
+            UnknownFieldEntry unkhfe(fieldType, utf8Len, utf8);
+            m_UHFL.push_back(unkhfe);
+            break;
+          }
 #endif
+          int rc = m_MapDBFilters.ImportFilterXMLFile(FPOOL_DATABASE, text.c_str(), _T(""),
+                                                      XSDFilename.c_str(),
+                                                      strErrors, m_pAsker);
+          if (rc != PWScore::SUCCESS) {
+            // Can't parse it - treat as an unknown field,
+            // Notify user that filter won't be available
+            stringT message;
+            LoadAString(message, IDSC_CANTPROCESSDBFILTERS);
+            if (m_pReporter != nullptr)
+              (*m_pReporter)(message);
+            pws_os::Trace(L"Error while parsing header filters.\n\tData: %ls\n\tErrors: %ls\n",
+                          text.c_str(), strErrors.c_str());
+            UnknownFieldEntry unkhfe(fieldType, utf8Len, utf8);
+            m_UHFL.push_back(unkhfe);
+          }
+        }
+        break;
+
       case HDR_RUE:
         {
           if (utf8 != nullptr) utf8[utf8Len] = '\0';
