@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -59,9 +59,8 @@ static void xormem(unsigned char *mem1, const unsigned char *mem2, int length)
 #endif
 void trashMemory(void *buffer, size_t length)
 {
-  ASSERT(buffer != nullptr);
   // {kjp} no point in looping around doing nothing is there?
-  if (length > 0) {
+  if (buffer != nullptr && length > 0) {
     std::memset(buffer, 0x55, length);
     std::memset(buffer, 0xAA, length);
     std::memset(buffer,    0, length);
@@ -158,37 +157,68 @@ void GenRandhash(const StringX &a_passkey,
   keyHash.Final(a_randhash);
 }
 
-size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length, unsigned char type,
-                 Fish *Algorithm, unsigned char *cbcbuffer)
+
+size_t _writecbc(FILE* fp, const unsigned char* buffer, size_t length, unsigned char type,
+  Fish* Algorithm, unsigned char* cbcbuffer)
 {
-  const unsigned int BS = Algorithm->GetBlockSize();
   size_t numWritten = 0;
 
+
+
+  // First encrypt and write the length of the buffer and type
+  numWritten = _writecbc1st(fp, &buffer, &length, type, Algorithm, cbcbuffer);
+  // now write the actual data
+  numWritten += _writecbcRest(fp, buffer, length, Algorithm, cbcbuffer);
+
+  return numWritten;
+}
+
+
+size_t _writecbc1st(FILE* fp, const unsigned char** buffer, size_t *length, unsigned char type,
+  Fish* Algorithm, unsigned char* cbcbuffer, bool isAboveThreshold)
+{
+  /**
+   * Write the length, type and perhaps a few bytes of data.
+   *
+   * aboveThreshold means the length value doesn't fit into 4 bytes (passed explicitly
+   * for testing with sub-gigabyte test data)
+   * In this case we just store the length as 8 bytes and the type, not bothering with the
+   * extra few bytes in the length block, since this is an optimization for small amounts of data,
+   * irrelevant for large lengths.
+   */
+  size_t numWritten = 0;
+  const unsigned int BS = Algorithm->GetBlockSize();
   // some trickery to avoid new/delete
   unsigned char block1[16];
 
-  unsigned char *curblock = nullptr;
+  unsigned char* curblock = nullptr;
   ASSERT(BS <= sizeof(block1)); // if needed we can be more sophisticated here...
 
-  // First encrypt and write the length of the buffer
   curblock = block1;
   // Fill unused bytes of length with random data, to make
   // a dictionary attack harder
   PWSrand::GetInstance()->GetRandomData(curblock, BS);
-  // block length overwrites 4 bytes of the above randomness.
-  putInt32(curblock, static_cast<int32>(length));
+
+  if (!isAboveThreshold) {
+    // block length overwrites 4 bytes of the above randomness.
+    putInt32(curblock, static_cast<int32>(*length));
+  } else {
+    ASSERT(BS >= 16);
+    memcpy(curblock, length, sizeof(size_t));
+  }
 
   // following new for format 2.0 - lengthblock bytes 4-7 were unused before.
-  curblock[sizeof(int32)] = type;
 
-  if (BS == 16) {
+  curblock[isAboveThreshold ? sizeof(size_t) : sizeof(int32)] = type;
+
+  if (BS == 16 && !isAboveThreshold) {
     // In this case, we've too many (11) wasted bytes in the length block
     // So we store actual data there:
     // (11 = BlockSize - 4 (length) - 1 (type)
-    const size_t len1 = (length > 11) ? 11 : length;
-    memcpy(curblock + 5, buffer, len1);
-    length -= len1;
-    buffer += len1;
+    const size_t len1 = (*length > 11) ? 11 : *length;
+    memcpy(curblock + 5, *buffer, len1);
+    *length -= len1;
+    *buffer += len1;
   }
 
   xormem(curblock, cbcbuffer, BS); // do the CBC thing
@@ -200,14 +230,12 @@ size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length, unsigned
     trashMemory(curblock, BS);
     throw(EIO);
   }
-
-  numWritten += _writecbc(fp, buffer, length, Algorithm, cbcbuffer);
-
   trashMemory(curblock, BS);
   return numWritten;
 }
 
-size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length,
+
+size_t _writecbcRest(FILE *fp, const unsigned char *buffer, size_t length,
                  Fish *Algorithm, unsigned char *cbcbuffer)
 {
   // Doesn't write out length, just CBC's the data, padding with randomness
@@ -257,6 +285,32 @@ size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length,
   return numWritten;
 }
 
+
+size_t readcbc1st(FILE *fp, size_t &record_size, Fish *Algorithm, unsigned char *cbcbuffer, bool isAboveThreshold)
+{
+  const unsigned int BS = Algorithm->GetBlockSize();
+  unsigned char* ctblock = new unsigned char[2 * BS];
+  memset(ctblock, 0, 2 * BS);
+  unsigned char* ptblock = ctblock + BS; // too lazy to allocate twice...
+
+  if (fread(ctblock, 1, BS, fp) != BS) {
+    delete[] ctblock;
+    record_size = 0;
+    return 0;
+  }
+
+  Algorithm->Decrypt(ctblock, ptblock);
+  xormem(ptblock, cbcbuffer, BS);
+  memcpy(cbcbuffer, ctblock, BS);
+
+  if (isAboveThreshold)
+    memcpy(&record_size, ptblock, sizeof(size_t));
+  else
+    record_size = getInt32(ptblock);
+  delete[] ctblock;
+  return BS;
+}
+
 /*
 * Reads an encrypted record into buffer.
 * The first block of the record contains the encrypted record length
@@ -438,12 +492,10 @@ StringX PWSUtil::ConvertToDateTimeString(const time_t &t, TMC result_format)
                 _T("%Y-%m-%dT%H:%M:%S"), st);
       break;
     case TMC_LOCALE:
-      setlocale(LC_TIME, "");
       _tcsftime(datetime_str, sizeof(datetime_str) / sizeof(datetime_str[0]),
                 _T("%c"), st);
       break;
     case TMC_LOCALE_DATE_ONLY:
-      setlocale(LC_TIME, "");
       _tcsftime(datetime_str, sizeof(datetime_str) / sizeof(datetime_str[0]),
                 _T("%x"), st);
       break;
@@ -752,6 +804,11 @@ unsigned int GetStringBufSize(const TCHAR *fmt, va_list args)
     nBytes = _vstprintf_s(buffer, len, fmt, ar);
     va_end(ar);//after using args we should reset list
     va_copy(ar, args);
+    /*
+     * If 'nBytes' is zero due to an empty format string,
+     * it would result in an endless memory-consuming loop.
+     */
+    ASSERT(nBytes != 0);
     if (nBytes++ > 0) {
       len = nBytes;
       break;
