@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -24,6 +24,8 @@
 #include <sstream>
 #include <stdarg.h>
 
+class Fish;
+
 // For V1V2 and file encryption, NOT for V3 and later:
 #define SaltLength 20
 #define StuffSize 10
@@ -50,6 +52,8 @@ extern void GenRandhash(const StringX &passkey,
                         const unsigned char *m_randstuff,
                         unsigned char *m_randhash);
 
+extern size_t readcbc1st(FILE* fp, size_t& record_size, Fish* Algorithm, unsigned char* cbcbuffer, bool isAboveThreshold = false);
+
 // buffer is allocated by _readcbc, *** delete[] is responsibility of caller ***
 extern size_t _readcbc(FILE *fp, unsigned char * &buffer,
                        size_t &buffer_len,
@@ -63,10 +67,17 @@ extern size_t _readcbc(FILE *fp, unsigned char *buffer,
                        const size_t buffer_len, Fish *Algorithm,
                        unsigned char *cbcbuffer);
 
-// _writecbc will throw(EIO) iff a write fail occurs!
-extern size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length,
-                        unsigned char type, Fish *Algorithm,
-                        unsigned char *cbcbuffer);
+// _writecbc* will throw(EIO) iff a write fail occurs!
+// version used to write records:
+extern size_t _writecbc(FILE* fp, const unsigned char* buffer, size_t length,
+  unsigned char type, Fish* Algorithm, unsigned char* cbcbuffer);
+
+// externalized implementation of above for whole-file encryption:
+extern size_t _writecbc1st(FILE* fp, const unsigned char** buffer, size_t* length, unsigned char type,
+  Fish* Algorithm, unsigned char* cbcbuffer, bool isAboveThreshold = false);
+extern size_t _writecbcRest(FILE* fp, const unsigned char* buffer, size_t length,
+  Fish* Algorithm, unsigned char* cbcbuffer);
+
 
 // typeless version for V4 content:
 extern size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length,
