@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -8,8 +8,6 @@
 
 #include "XMLDefs.h"    // Required if testing "USE_XML_LIBRARY"
 
-#ifdef USE_XML_LIBRARY
-
 #include "XMLFileValidation.h"
 #include "XMLFileHandlers.h"
 
@@ -1035,5 +1033,3 @@ void XMLFileHandlers::AddDBPreferences()
   // Copy over preferences from XML input to this DB (only if DB was empty before import)
   PWSprefs::GetInstance()->UpdateFromCopyPrefs(PWSprefs::ptDatabase);
 }
-
-#endif /* USE_XML_LIBRARY */
