@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -32,6 +32,13 @@ namespace pws_os {
   extern bool    IsWindows81OrGreater();
   extern bool    IsWindows10OrGreater();
 #endif
+
+#ifdef _WIN32
+inline bool IsWindows() {return true;}
+#else
+inline bool IsWindows() {return false;}
+#endif
+
 }
 #endif /* __ENV_H */
 //-----------------------------------------------------------------------------
