@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -23,6 +23,7 @@
 #include "AboutDlg.h"
 #include "version.h"
 #include "PasswordSafeFrame.h"
+#include "PWSafeApp.h"
 #include "core/CheckVersion.h"
 
 #ifdef __WXMSW__
@@ -62,27 +63,14 @@ const cstringT AboutDlg::s_URL_VERSION   =  "https://pwsafe.org/latest.xml";
  * AboutDlg constructors
  */
 
-AboutDlg::AboutDlg()
+AboutDlg::AboutDlg(wxWindow *parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
 {
-  Init();
-}
-
-AboutDlg::AboutDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
-{
-  Init();
-  Create(parent, id, caption, pos, size, style);
+  wxASSERT(!parent || parent->IsTopLevel());
 
   // Print version information on standard output which might be useful for error reports.
   pws_os::Trace(GetLibWxVersion().wc_str());
   pws_os::Trace(GetLibCurlVersion().wc_str());
-}
 
-/*!
- * AboutDlg creator
- */
-
-bool AboutDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
-{
   SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
   wxDialog::Create( parent, id, caption, pos, size, style );
 
@@ -98,27 +86,11 @@ bool AboutDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption,
     }
   }
   Centre();
-  return true;
 }
 
-/*!
- * AboutDlg destructor
- */
-
-AboutDlg::~AboutDlg()
+AboutDlg* AboutDlg::Create(wxWindow *parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
 {
-////@begin AboutDlg destruction
-////@end AboutDlg destruction
-}
-
-/*!
- * Member initialization
- */
-
-void AboutDlg::Init()
-{
-  m_VersionStatus = nullptr;
-  m_CurlHandle = nullptr;
+  return new AboutDlg(parent, id, caption, pos, size, style);
 }
 
 /*!
@@ -171,7 +143,7 @@ void AboutDlg::CreateControls()
   wxStaticText* licenseStaticTextEnd = new wxStaticText(aboutDialog, wxID_STATIC, _("See LICENSE for open source details."), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
   rightSizer->Add(licenseStaticTextEnd, 0, wxALIGN_LEFT|wxALL, 5);
 
-  wxStaticText* copyrightStaticText = new wxStaticText(aboutDialog, wxID_STATIC, _("Copyright (c) 2003-2020 Rony Shapiro"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
+  wxStaticText* copyrightStaticText = new wxStaticText(aboutDialog, wxID_STATIC, _("Copyright (c) 2003-2022 Rony Shapiro"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
   rightSizer->Add(copyrightStaticText, 0, wxALIGN_LEFT|wxALL, 5);
 
   m_VersionStatus = new wxTextCtrl(aboutDialog, ID_TEXTCTRL, wxT("\n\n"), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxNO_BORDER);
@@ -423,7 +395,7 @@ wxString AboutDlg::GetLibWxVersion()
  */
 bool AboutDlg::CheckDatabaseStatus()
 {
-  PasswordSafeFrame *pwsafe = static_cast<PasswordSafeFrame *>(GetParent());
+  auto pwsafe = wxGetApp().GetPasswordSafeFrame();
 
   if (!pwsafe->IsClosed()) {
 
@@ -445,21 +417,16 @@ bool AboutDlg::CheckDatabaseStatus()
     // Notify PasswordSafeFrame to close database.
     // If there are any unsaved changes PasswordSafeFrame 
     // will prompt the user to save them.
-    wxCommandEvent closeEvent(wxEVT_COMMAND_MENU_SELECTED, wxID_CLOSE);
-#if wxCHECK_VERSION(2,9,0)
-    pwsafe->GetEventHandler()->ProcessEvent(closeEvent);
-#else
-    pwsafe->ProcessEvent(closeEvent);
-#endif
-
-    // Check database once again, because user could have cancelled to save changes.
-    if (!pwsafe->IsClosed()) {
-      return false;
-    }
+    pwsafe->CloseDB([](bool closed) {
+      if (closed) {
+        // database closed, reopen dialog and check
+        DestroyWrapper<AboutDlg> wrapper(wxGetApp().GetPasswordSafeFrame());
+        wrapper.Get()->ShowAndCheckForUpdate();
+      }
+    });
+    return false; // stop here, callback will restart
   }
 
-  // Update UI accordingly to show user that database is closed.
-  pwsafe->Update();
   ASSERT(pwsafe->GetNumEntries() == 0);
 
   // Now, database is closed.
@@ -705,3 +672,9 @@ void AboutDlg::OnDownloadCompleted(wxThreadEvent& event)
 void AboutDlg::OnVisitSiteClicked(wxHyperlinkEvent& WXUNUSED(event)) {
   wxLaunchDefaultBrowser(s_URL_HOME);
 }
+
+
+int AboutDlg::ShowAndCheckForUpdate() {
+  CallAfter(&AboutDlg::CheckNewVersion);
+  return ShowModal();
+}
