@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -32,7 +32,7 @@ public:
 #endif
 private:
   Clipboard();
-  ~Clipboard() {};
+  ~Clipboard() {}
   Clipboard(const Clipboard &);
   Clipboard &operator=(const Clipboard &);
 
