@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -27,12 +27,12 @@
 #include "wxUtilities.h"
 
 //(*IdInit(CreateShortcutDlg)
-const long CreateShortcutDlg::ID_COMBOBOX1 = wxNewId();
-const long CreateShortcutDlg::ID_TEXTCTRL1 = wxNewId();
-const long CreateShortcutDlg::ID_TEXTCTRL2 = wxNewId();
-const long CreateShortcutDlg::ID_STATICTEXT7 = wxNewId();
-const long CreateShortcutDlg::ID_STATICTEXT8 = wxNewId();
-const long CreateShortcutDlg::ID_STATICTEXT9 = wxNewId();
+const wxWindowID CreateShortcutDlg::ID_COMBOBOX1 = wxNewId();
+const wxWindowID CreateShortcutDlg::ID_TEXTCTRL1 = wxNewId();
+const wxWindowID CreateShortcutDlg::ID_TEXTCTRL2 = wxNewId();
+const wxWindowID CreateShortcutDlg::ID_STATICTEXT7 = wxNewId();
+const wxWindowID CreateShortcutDlg::ID_STATICTEXT8 = wxNewId();
+const wxWindowID CreateShortcutDlg::ID_STATICTEXT9 = wxNewId();
 //*)
 
 /*!
@@ -48,35 +48,20 @@ IMPLEMENT_CLASS( CreateShortcutDlg, wxDialog )
 BEGIN_EVENT_TABLE( CreateShortcutDlg, wxDialog )
 
   EVT_BUTTON( wxID_OK, CreateShortcutDlg::OnOk )
-
+  EVT_CLOSE( CreateShortcutDlg::OnClose )
+  EVT_BUTTON( wxID_CANCEL, CreateShortcutDlg::OnCancelClick )
 END_EVENT_TABLE()
 
 /*!
  * CreateShortcutDlg constructors
  */
 
-CreateShortcutDlg::CreateShortcutDlg(wxWindow* parent, PWScore &core, CItemData *base)
+CreateShortcutDlg::CreateShortcutDlg(wxWindow *parent, PWScore &core, CItemData *base)
 : m_Core(core), m_Base(base)
 {
   ASSERT(m_Base != nullptr);
-  Init();
-  Create(parent);
-}
+  wxASSERT(!parent || parent->IsTopLevel());
 
-/*!
- * CreateShortcutDlg destructor
- */
-
-CreateShortcutDlg::~CreateShortcutDlg()
-{
-}
-
-/*!
- * CreateShortcutDlg creator
- */
-
-bool CreateShortcutDlg::Create(wxWindow* parent)
-{
   SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
   wxDialog::Create(parent, wxID_ANY, _("Create Shortcut"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER);
 
@@ -89,25 +74,46 @@ bool CreateShortcutDlg::Create(wxWindow* parent)
   SetValidators();
   UpdateControls();
   ItemFieldsToDialog();
-  return true;
+}
+
+CreateShortcutDlg* CreateShortcutDlg::Create(wxWindow *parent, PWScore &core, CItemData *base)
+{
+  return new CreateShortcutDlg(parent, core, base);
 }
 
 void CreateShortcutDlg::ItemFieldsToDialog()
 {
   // Populate the combo box
   std::vector<stringT> allGroupNames;
+  wxSize actSize = m_ComboBoxShortcutGroup->GetSize(), oldSize = actSize, borderSize = m_ComboBoxShortcutGroup->GetWindowBorderSize();
+  wxScreenDC dc;
+  wxCoord width, height, border = (borderSize.GetWidth() * 2) + 2;
+  
+  dc.SetFont(m_ComboBoxShortcutGroup->GetFont());
+  
+  m_Core.GetAllGroups(allGroupNames, true); // Also allow short cut in empty groups
 
-  m_Core.GetAllGroups(allGroupNames, false);
-
+  m_ComboBoxShortcutGroup->Append(""); // Also allow selection of emtpy group
   for (auto const& groupName : allGroupNames) {
     m_ComboBoxShortcutGroup->Append(groupName);
+    dc.GetTextExtent(groupName, &width, &height);
+    width += border; // Align
+    if(width > actSize.GetWidth()) actSize.SetWidth(width);
   }
-
+  if(actSize.GetWidth() != oldSize.GetWidth()) {
+    GetSize(&width, &height);
+    width += actSize.GetWidth() - oldSize.GetWidth();
+    int displayWidth, displayHight;
+    ::wxDisplaySize(&displayWidth, &displayHight);
+    if(width > displayWidth) width = displayWidth;
+    SetSize(width, height);
+  }
+  
   if (m_Base != nullptr) {
     m_BaseEntryTitle = stringx2std(m_Base->GetTitle());
     m_BaseEntryUsername = stringx2std(m_Base->GetUser());
 
-    m_ShortcutTitle = _("Shortcut to ") + m_BaseEntryTitle;
+    m_ShortcutTitle = GetDefaultShortcutTitle();
     m_ShortcutUsername = m_BaseEntryUsername;
 
     if (m_Base->IsGroupSet() && !(m_Base->GetGroup().empty())) {
@@ -117,6 +123,7 @@ void CreateShortcutDlg::ItemFieldsToDialog()
 
       if (position != wxNOT_FOUND) {
         m_ShortcutGroup = m_BaseEntryGroup;
+        m_ComboBoxShortcutGroup->SetSelection(position);
       }
       else {
         m_ShortcutGroup = wxEmptyString;
@@ -131,9 +138,10 @@ void CreateShortcutDlg::ItemFieldsToDialog()
     m_ShortcutTitle = wxEmptyString;
     m_ShortcutUsername = wxEmptyString;
 
-    m_BaseEntryGroup = _("N/A");
-    m_BaseEntryTitle = _("N/A");
-    m_BaseEntryUsername = _("N/A");
+    // set base values to empty too/ to siplify change detection
+    m_BaseEntryGroup = wxEmptyString;
+    m_BaseEntryTitle = wxEmptyString;
+    m_BaseEntryUsername = wxEmptyString;
   }
 }
 
@@ -157,20 +165,6 @@ void CreateShortcutDlg::UpdateControls()
   }
 }
 
-/*!
- * Member initialisation
- */
-
-void CreateShortcutDlg::Init()
-{
-  m_ComboBoxShortcutGroup = nullptr;
-  m_TextCtrlShortcutTitle = nullptr;
-  m_TextCtrlShortcutUsername = nullptr;
-  m_StaticTextBaseEntryGroup = nullptr;
-  m_StaticTextBaseEntryTitle = nullptr;
-  m_StaticTextBaseEntryUsername = nullptr;
-}
-
 /*!
  * Control creation for CreateShortcutDlg
  */
@@ -307,3 +301,36 @@ void CreateShortcutDlg::OnOk(wxCommandEvent& WXUNUSED(event))
   }
   EndModal(wxID_OK);
 }
+
+bool CreateShortcutDlg::SyncAndQueryCancel(bool showDialog) {
+  // when edit forbidden, allow cancel without additional checks
+  if (m_Core.IsReadOnly()) {
+    return true;
+  }
+  return QueryCancelDlg::SyncAndQueryCancel(showDialog);
+}
+
+uint32_t CreateShortcutDlg::GetChanges() const
+{
+  uint32_t changes = Changes::None;
+  if (m_ShortcutTitle != GetDefaultShortcutTitle()) {
+      changes |= Changes::Title;
+  }
+  if (m_ShortcutUsername != m_BaseEntryUsername) {
+    changes |= Changes::User;
+  }
+    
+  if (m_ShortcutGroup != m_BaseEntryGroup) {
+    changes |= Changes::Group;
+  }
+  return changes;
+}
+
+bool CreateShortcutDlg::IsChanged() const {
+  return GetChanges() != Changes::None;
+}
+
+wxString CreateShortcutDlg::GetDefaultShortcutTitle() const
+{
+  return _("Shortcut to ") + m_BaseEntryTitle;
+}
