@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -18,13 +18,13 @@
  */
 
 #include <wx/choice.h>
-#include <wx/dialog.h>
 #include <wx/sizer.h>
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
 
 #include "core/ItemData.h"
 #include "core/PWScore.h"
+#include "QueryCancelDlg.h"
 
 /*!
  * Forward declarations
@@ -37,23 +37,21 @@
  * CreateShortcutDlg class declaration
  */
 
-class CreateShortcutDlg : public wxDialog
+class CreateShortcutDlg : public QueryCancelDlg
 {
   DECLARE_CLASS(CreateShortcutDlg)
   DECLARE_EVENT_TABLE()
 
 public:
-  /// Constructors
-  CreateShortcutDlg(wxWindow* parent, PWScore &core, CItemData *base);
+  /// Creation
+  static CreateShortcutDlg* Create(wxWindow *parent, PWScore &core, CItemData *base);
 
   /// Destructor
-  virtual ~CreateShortcutDlg();
+  virtual ~CreateShortcutDlg() = default;
 
-  /// Creation
-  bool Create(wxWindow* parent);
-
-  /// Initialises member variables
-  void Init();
+protected:
+  /// Constructors
+  CreateShortcutDlg(wxWindow *parent, PWScore &core, CItemData *base);
 
   /// Creates the controls and sizers
   void CreateControls();
@@ -72,28 +70,41 @@ private:
   void ItemFieldsToDialog();
   void SetValidators();
   void UpdateControls();
+  bool SyncAndQueryCancel(bool showDialog) override;
 
+  enum Changes : uint32_t {
+    None = 0,
+    Group = 1u,
+    Title = 1u << 1,
+    User = 1u << 2,
+  };
+  
+  uint32_t GetChanges() const;
+  bool IsChanged() const override;
+
+  wxString GetDefaultShortcutTitle() const;
+  
   //(*Handlers(CreateShortcutDlg)
   void OnOk(wxCommandEvent& event);
   //*)
 
   //(*Identifiers(CreateShortcutDlg)
-  static const long ID_COMBOBOX1;
-  static const long ID_TEXTCTRL1;
-  static const long ID_TEXTCTRL2;
-  static const long ID_STATICTEXT7;
-  static const long ID_STATICTEXT8;
-  static const long ID_STATICTEXT9;
+  static const wxWindowID ID_COMBOBOX1;
+  static const wxWindowID ID_TEXTCTRL1;
+  static const wxWindowID ID_TEXTCTRL2;
+  static const wxWindowID ID_STATICTEXT7;
+  static const wxWindowID ID_STATICTEXT8;
+  static const wxWindowID ID_STATICTEXT9;
   //*)
 
   //(*Declarations(CreateShortcutDlg)
-  wxComboBox* m_ComboBoxShortcutGroup;
-  wxTextCtrl* m_TextCtrlShortcutTitle;
-  wxTextCtrl* m_TextCtrlShortcutUsername;
+  wxComboBox* m_ComboBoxShortcutGroup = nullptr;
+  wxTextCtrl* m_TextCtrlShortcutTitle = nullptr;
+  wxTextCtrl* m_TextCtrlShortcutUsername = nullptr;
 
-  wxStaticText* m_StaticTextBaseEntryGroup;
-  wxStaticText* m_StaticTextBaseEntryTitle;
-  wxStaticText* m_StaticTextBaseEntryUsername;
+  wxStaticText* m_StaticTextBaseEntryGroup = nullptr;
+  wxStaticText* m_StaticTextBaseEntryTitle = nullptr;
+  wxStaticText* m_StaticTextBaseEntryUsername = nullptr;
   //*)
 
   wxString m_ShortcutGroup;
@@ -105,7 +116,7 @@ private:
   wxString m_BaseEntryUsername;
 
   PWScore &m_Core;
-  CItemData *m_Base;
+  CItemData *m_Base = nullptr;
 };
 
 #endif // _CREATESHORTCUTDLG_H_
