@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -23,6 +23,7 @@
 #include "GridCtrl.h"
 #include "GridTable.h"
 #include "PasswordSafeFrame.h" // for DispatchDblClickAction()
+#include "PWSafeApp.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/msvcrt.h>
@@ -364,13 +365,6 @@ void GridCtrl::RefreshItemField(const pws_os::CUUID& uuid, CItemData::FieldType
   }
 }
 
-struct moveup : public std::binary_function<UUIDRowMapT::value_type, int, void> {
-  void operator()(UUIDRowMapT::value_type& v, int rowDeleted) const {
-    if (v.second > rowDeleted)
-      v.second = v.second - 1;
-  }
-};
-
 void GridCtrl::Remove(const CUUID &uuid)
 {
   auto iter = m_uuid_map.find(uuid);
@@ -392,7 +386,7 @@ void GridCtrl::Remove(const CUUID &uuid)
 
     //subtract the row values of all entries in uuid map if it is greater
     //than the row we just deleted
-    std::for_each(m_uuid_map.begin(), m_uuid_map.end(), std::bind2nd(moveup(), row));
+    std::for_each(m_uuid_map.begin(), m_uuid_map.end(), [row](UUIDRowMapT::value_type &v){if (v.second > row) v.second -= 1;} );
 
     //delete the item itself
     m_uuid_map.erase(iter);
@@ -489,7 +483,7 @@ void GridCtrl::OnCellRightClick( wxGridEvent& evt )
   //
   SetGridCursor(evt.GetRow(), evt.GetCol());
   SelectRow(evt.GetRow());
-  dynamic_cast<PasswordSafeFrame *>(GetParent())->OnContextMenu(GetItem(evt.GetRow()));
+  wxGetApp().GetPasswordSafeFrame()->OnContextMenu(GetItem(evt.GetRow()));
 }
 
 /*!
@@ -502,7 +496,7 @@ void GridCtrl::OnContextMenu( wxContextMenuEvent& evt )
   if ( pos == wxDefaultPosition ) { //sent from keyboard?
     const int row = GetGridCursorRow();
     SelectRow(row);
-    dynamic_cast<PasswordSafeFrame *>(GetParent())->OnContextMenu(GetItem(row));
+    wxGetApp().GetPasswordSafeFrame()->OnContextMenu(GetItem(row));
   }
   else { //sent from mouse.  I don't know how to convert the mouse coords to grid's row,column
     evt.Skip();
@@ -533,8 +527,7 @@ void GridCtrl::OnLeftDClick( wxGridEvent& evt )
 {
   CItemData *item = GetItem(evt.GetRow());
   if (item != nullptr)
-    dynamic_cast<PasswordSafeFrame *>(GetParent())->
-      DispatchDblClickAction(*item);
+    wxGetApp().GetPasswordSafeFrame()->DispatchDblClickAction(*item);
 }
 
  void GridCtrl::SelectItem(const CUUID & uuid)
@@ -585,7 +578,7 @@ void GridCtrl::OnSelectCell( wxGridEvent& evt )
 {
   CItemData *pci = GetItem(evt.GetRow());
 
-  dynamic_cast<PasswordSafeFrame *>(GetParent())->UpdateSelChanged(pci);
+  wxGetApp().GetPasswordSafeFrame()->UpdateSelChanged(pci);
 }
 
 void GridCtrl::SetFilterState(bool state)
@@ -620,7 +613,7 @@ void GridCtrl::OnMouseRightClick(wxMouseEvent& event)
   /* check whether a grid cell was hit */
   if (!HasGridCell(gridCellInfo)) {
     ClearSelection();
-    dynamic_cast<PasswordSafeFrame *>(GetParent())->OnContextMenu(nullptr);
+    wxGetApp().GetPasswordSafeFrame()->OnContextMenu(nullptr);
   }
   else {
     event.Skip();
