@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -67,7 +67,8 @@ void GuiInfo::SaveTreeViewInfo(TreeCtrl* tree)
     }
     else {
       m_treeTop.Clear();
-      wxFAIL_MSG(wxString(wxT("Tree item \'")) << tree->GetItemText(treeItem) << wxT("\' found with no children and no CItemData"));
+      // The following can happen after UNDO of XML Import - remove FAIL_MSG to do not stop in case of the problem...
+      // wxFAIL_MSG(wxString(wxT("Tree item \'")) << tree->GetItemText(treeItem) << wxT("\' found with no children and no CItemData"));
     }
   }
   else {
@@ -94,7 +95,6 @@ void GuiInfo::SaveTreeViewInfo(TreeCtrl* tree)
       }
       else {
         m_treeSelection.Clear();
-        wxFAIL_MSG(wxString(wxT("tree item \'")) << tree->GetItemText(selection) << wxT("\' found with no CItemData attached"));
       }
     }
   }
@@ -164,7 +164,7 @@ void RestoreTreeItem(TreeCtrl* tree, const string_or_uuid& val, TreeFunc func)
   wxTreeItemId id;
   switch(val.Type()) {
     case string_or_uuid::ItemType::NORMAL:
-      id = tree->Find(static_cast<pws_os::CUUID> (val));
+      id = tree->TreeCtrlBase::Find(static_cast<pws_os::CUUID> (val));
       break;
     case string_or_uuid::ItemType::GROUP:
       id = tree->Find(static_cast<wxString>(val), tree->GetRootItem());
