@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -52,12 +52,13 @@ IMPLEMENT_CLASS( ImportTextDlg, wxDialog )
 BEGIN_EVENT_TABLE( ImportTextDlg, wxDialog )
 END_EVENT_TABLE()
 
-ImportTextDlg::ImportTextDlg(wxWindow* parent) :  wxDialog(parent,
+ImportTextDlg::ImportTextDlg(wxWindow *parent, const wxString& filename) :  wxDialog(parent,
                                                             wxID_ANY,
                                                             _("Import Text Settings"),
                                                             wxDefaultPosition,
                                                             wxDefaultSize,
                                                             wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),
+                                                    filepath(filename),
                                                     delimiterComma(false),
                                                     delimiterSpace(false),
                                                     delimiterTab(true), //this is the pwsafe's default for export
@@ -67,16 +68,19 @@ ImportTextDlg::ImportTextDlg(wxWindow* parent) :  wxDialog(parent,
                                                     importUnderGroup(false),
                                                     importPasswordsOnly(false)
 {
+  wxASSERT(!parent || parent->IsTopLevel());
+
   //since the controls aren't direct children of the dialog but instead are
   //parented by wxCollipsiblePane, we need to validate recursively
-  SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);
+  SetExtraStyle(GetExtraStyle() | wxWS_EX_VALIDATE_RECURSIVELY);
   CreateControls();
 }
 
-ImportTextDlg::~ImportTextDlg()
+ImportTextDlg* ImportTextDlg::Create(wxWindow *parent, const wxString& filename)
 {
+  return new ImportTextDlg(parent, filename);
 }
-
+                                                                  
 wxCollapsiblePane* ImportTextDlg::CreateImportOptionsPane(wxBoxSizer* dlgSizer)
 {
   const wxSizerFlags Left = wxSizerFlags().Proportion(0).Border(wxLEFT, SideMargin);
@@ -108,7 +112,7 @@ wxBoxSizer* ImportTextDlg::CreateVerticalButtonSizer(long flags)
   wxBoxSizer* box = new wxBoxSizer(wxVERTICAL);
   box->AddSpacer(TopMargin);
 
-  long buttons[] = {wxID_OK, wxID_CANCEL, wxID_HELP};
+  wxWindowID buttons[] = {wxID_OK, wxID_CANCEL, wxID_HELP};
   for (size_t idx = 0; idx < NumberOf(buttons); ++idx) {
     if ((flags & buttons[idx]) == buttons[idx]){
       box->Add(new wxButton(this, buttons[idx]));
@@ -186,7 +190,7 @@ void ImportTextDlg::CreateControls()
   dlgSizer->Add(new wxStaticText(this, wxID_ANY, strPrompt), Left);
   dlgSizer->AddSpacer(RowSeparation);
   OpenFilePickerValidator validator(filepath);
-  dlgSizer->Add(new wxFilePickerCtrl(this, wxID_ANY, wxEmptyString,
+  dlgSizer->Add(new wxFilePickerCtrl(this, wxID_ANY, filepath,
                                           strPrompt, wildCards,
                                           wxDefaultPosition, wxDefaultSize,
                                           wxFLP_DEFAULT_STYLE | wxFLP_USE_TEXTCTRL,
