@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -24,6 +24,7 @@
 #include <wx/msw/msvcrt.h>
 #endif
 
+#include <wx/richmsgdlg.h>
 #include <wx/tokenzr.h>
 
 #include "core/PWSAuxParse.h"
@@ -65,17 +66,14 @@ void PasswordSafeFrame::DoEdit(CItemData item)
   if (!item.IsShortcut()) {
     bool isItemReadOnly = m_core.IsReadOnly() || item.IsProtected();
 
-    AddEditPropSheetDlg dialog(
-      this, m_core,
+    returnCode = ShowModalAndGetResult<AddEditPropSheetDlg>(this,
+      m_core,
       isItemReadOnly ? AddEditPropSheetDlg::SheetType::VIEW : AddEditPropSheetDlg::SheetType::EDIT,
       &item
     );
-
-    returnCode = dialog.ShowModal();
   }
   else {
-    EditShortcutDlg dialog(this, m_core, &item);
-    returnCode = dialog.ShowModal();
+    returnCode = ShowModalAndGetResult<EditShortcutDlg>(this, m_core, &item);
   }
 
   if (returnCode == wxID_OK) {
@@ -110,8 +108,8 @@ void PasswordSafeFrame::OnAddClick(wxCommandEvent& WXUNUSED(evt))
     selectedGroup = m_tree->GetItemGroup(selection);
   }
 
-  AddEditPropSheetDlg addDbox(this, m_core, AddEditPropSheetDlg::SheetType::ADD, nullptr, selectedGroup);
-  if (addDbox.ShowModal() == wxID_OK) {
+  int rc = ShowModalAndGetResult<AddEditPropSheetDlg>(this, m_core, AddEditPropSheetDlg::SheetType::ADD, nullptr, selectedGroup);
+  if (rc == wxID_OK) {
     UpdateStatusBar();
   }
 }
@@ -123,8 +121,9 @@ void PasswordSafeFrame::OnAddClick(wxCommandEvent& WXUNUSED(evt))
 void PasswordSafeFrame::OnEditClick(wxCommandEvent& WXUNUSED(evt))
 {
   CItemData *item = GetSelectedEntry();
-  if (item != nullptr)
-    DoEdit(*item);
+  if (item) {
+    CallAfter(&PasswordSafeFrame::DoEdit, *item);
+  }
 }
 
 /*!
@@ -136,32 +135,44 @@ void PasswordSafeFrame::OnDeleteClick(wxCommandEvent& WXUNUSED(evt))
   bool dontaskquestion = PWSprefs::GetInstance()->
     GetPref(PWSprefs::DeleteQuestion);
 
-  bool dodelete = true;
   int num_children = 0;
   // If tree view, check if group selected
   if (m_tree->IsShown()) {
     wxTreeItemId sel = m_tree->GetSelection();
-    num_children = static_cast<int>(m_tree->GetChildrenCount(sel));
+    if(m_tree->ItemIsGroup(sel)) {
+      num_children = static_cast<int>(m_tree->GetChildrenCount(sel));
+    }
+    else {
+      num_children = 0;
+    }
     if (num_children > 0) // ALWAYS confirm group delete
       dontaskquestion = false;
   }
+  CallAfter(&PasswordSafeFrame::DoDeleteItems, !dontaskquestion, num_children);
+}
 
+void PasswordSafeFrame::DoDeleteItems(bool askConfirmation, int num_children)
+{  
+  bool dodelete = true;
   //Confirm whether to delete the item
-  if (!dontaskquestion) {
-    DeleteConfirmationDlg deleteDlg(this, num_children);
-    deleteDlg.SetConfirmdelete(PWSprefs::GetInstance()->GetPref(PWSprefs::DeleteQuestion));
-    int rc = deleteDlg.ShowModal();
+  if (askConfirmation) {
+    DestroyWrapper<DeleteConfirmationDlg> deleteDlgWrapper(this, num_children);
+    DeleteConfirmationDlg* deleteDlg = deleteDlgWrapper.Get();
+    deleteDlg->SetConfirmdelete(PWSprefs::GetInstance()->GetPref(PWSprefs::DeleteQuestion));
+    int rc = deleteDlg->ShowModal();
     if (rc != wxID_YES) {
       dodelete = false;
     }
-    PWSprefs::GetInstance()->SetPref(PWSprefs::DeleteQuestion, deleteDlg.GetConfirmdelete());
+    if (!IsCloseInProgress()) {
+      PWSprefs::GetInstance()->SetPref(PWSprefs::DeleteQuestion, deleteDlg->GetConfirmdelete());
+    }
   }
 
   if (dodelete) {
     Command *doit = nullptr;
     CItemData *item = GetSelectedEntry();
     if (item != nullptr) {
-      doit = Delete(item);
+      doit = DeleteItem(item);
     } else if (m_tree->GetSelection() != m_tree->GetRootItem()) {
       doit = Delete(m_tree->GetSelection());
     }
@@ -170,31 +181,37 @@ void PasswordSafeFrame::OnDeleteClick(wxCommandEvent& WXUNUSED(evt))
   } // dodelete
 }
 
-Command *PasswordSafeFrame::Delete(CItemData *pci)
+Command *PasswordSafeFrame::DeleteItem(CItemData *pci, wxTreeItemId root)
 {
   ASSERT(pci != nullptr);
+  // Alias in same group, which will be deleted, are not given warning for
+  StringX sxGroup = L"";
+  if(root)
+    sxGroup = tostringx(m_tree->GetItemGroup(root));
   // ConfirmDelete asks for user confirmation
   // when deleting a shortcut or alias base.
   // Otherwise it just returns true
-  if (m_core.ConfirmDelete(pci))
+  if (m_core.ConfirmDelete(pci, sxGroup))
     return DeleteEntryCommand::Create(&m_core, *pci);
   else
     return nullptr;
 }
 
-Command *PasswordSafeFrame::Delete(wxTreeItemId tid)
+Command *PasswordSafeFrame::Delete(wxTreeItemId tid, wxTreeItemId root)
 {
   // Called for deleting a group
   // Recursively build the appropriate multi-command
 
   if (!tid) return nullptr;
+  if (!root)
+    root = tid;
   MultiCommands *retval = MultiCommands::Create(&m_core);
-  if (m_tree->GetChildrenCount(tid) > 0) {
+  if (m_tree->ItemIsGroup(tid) && (m_tree->GetChildrenCount(tid) > 0)) {
     wxTreeItemIdValue cookie;
     wxTreeItemId ti = m_tree->GetFirstChild(tid, cookie);
 
     while (ti.IsOk()) {
-      Command *delCmd = Delete(ti);
+      Command *delCmd = Delete(ti, root);
       if (delCmd != nullptr)
         retval->Add(delCmd);
       ti = m_tree->GetNextChild(tid, cookie);
@@ -211,7 +228,7 @@ Command *PasswordSafeFrame::Delete(wxTreeItemId tid)
   } else { // leaf
     CItemData *leaf = m_tree->GetItem(tid);
     if (leaf != nullptr) {
-      Command *delLeafCmd = Delete(leaf); // gets user conf. if needed
+      Command *delLeafCmd = DeleteItem(leaf, root); // gets user conf. if needed
       if (delLeafCmd != nullptr)
         retval->Add(delLeafCmd);
     }
@@ -238,7 +255,10 @@ Command *PasswordSafeFrame::Delete(wxTreeItemId tid)
 
 void PasswordSafeFrame::OnFindClick(wxCommandEvent& WXUNUSED(evt))
 {
+  wxASSERT(m_search);
   m_search->Activate();
+  ShowSearchBar();
+  m_search->SetFocusIntoEditField();
 }
 
 /*!
@@ -364,6 +384,21 @@ void PasswordSafeFrame::OnCopyPasswordClick(wxCommandEvent& evt)
 
 void PasswordSafeFrame::DoCopyPassword(CItemData &item)
 {
+  if (PWSprefs::GetInstance()->GetPref(PWSprefs::DontAskQuestion)) {
+    wxRichMessageDialog dialog(this, 
+      _("Pressing OK will copy the password of the selected item\nto the clipboard. The clipboard will be securely cleared\nwhen Password Safe is closed.\n\nPressing Cancel stops the password from being copied."), 
+      _("Clear Clipboard"), 
+      wxOK | wxCANCEL | wxICON_INFORMATION);
+
+    dialog.ShowCheckBox(_("&Don't remind me again"));
+
+    if(dialog.ShowModal() == wxID_CANCEL) {
+      return;
+    }
+
+    PWSprefs::GetInstance()->SetPref(PWSprefs::DontAskQuestion, !dialog.IsCheckBoxChecked());
+  }
+  
   if (!item.IsDependent())
     Clipboard::GetInstance()->SetData(item.GetPassword());
   else {
@@ -509,6 +544,11 @@ void PasswordSafeFrame::OnViewAttachment(wxCommandEvent& WXUNUSED(evt))
     return;
   }
 
+  CallAfter(&PasswordSafeFrame::DoViewAttachment, item);
+}
+
+void PasswordSafeFrame::DoViewAttachment(CItemData* item)
+{
   ASSERT(m_core.HasAtt(item->GetAttUUID()));
 
   CItemAtt itemAttachment = m_core.GetAtt(item->GetAttUUID());
@@ -532,10 +572,11 @@ void PasswordSafeFrame::OnViewAttachment(wxCommandEvent& WXUNUSED(evt))
     return;
   }
 
-  ViewAttachmentDlg viewAttachmentDlg(this);
+  DestroyWrapper<ViewAttachmentDlg> viewAttachmentDlgWrapper(this);
+  ViewAttachmentDlg *viewAttachmentDlg = viewAttachmentDlgWrapper.Get();
 
-  if (viewAttachmentDlg.LoadImage(itemAttachment)) {
-    viewAttachmentDlg.ShowModal();
+  if (viewAttachmentDlg->LoadImage(itemAttachment)) {
+    viewAttachmentDlg->ShowModal();
   }
   else {
     wxMessageDialog(
@@ -566,13 +607,16 @@ void PasswordSafeFrame::OnCreateShortcut(wxCommandEvent& WXUNUSED(event))
 {
   CItemData* item = GetSelectedEntry();
   if (item && !item->IsDependent()) {
-    CreateShortcutDlg dlg(this, m_core, item);
-    int rc = dlg.ShowModal();
-    if (rc == wxID_OK) {
-      UpdateAccessTime(*item);
-      Show(true);
-      UpdateStatusBar();
-    }
+    CallAfter(&PasswordSafeFrame::DoCreateShortcut, item);
+  }
+}
+
+void PasswordSafeFrame::DoCreateShortcut(CItemData* item)
+{
+  if (ShowModalAndGetResult<CreateShortcutDlg>(this, m_core, item) == wxID_OK) {
+    UpdateAccessTime(*item);
+    Show(true);
+    UpdateStatusBar();
   }
 }
 
@@ -1030,11 +1074,12 @@ void PasswordSafeFrame::DoEmail(CItemData& item )
   }
 }
 
-void PasswordSafeFrame::DoPasswordSubset(CItemData& item )
+void PasswordSafeFrame::DoPasswordSubset(CItemData* item)
 {
-  PasswordSubsetDlg psDlg(this, item.GetPassword());
-  psDlg.ShowModal();
-  UpdateAccessTime(item);
+  if (item) {
+    ShowModalAndGetResult<PasswordSubsetDlg>(this, item->GetPassword());
+    UpdateAccessTime(*item);
+  }
 }
 
 void PasswordSafeFrame::OnUndo(wxCommandEvent& evt)
@@ -1055,8 +1100,9 @@ void PasswordSafeFrame::OnPasswordSubset(wxCommandEvent &evt)
 {
   CItemData rueItem;
   CItemData* item = GetSelectedEntry(evt, rueItem);
-  if (item != nullptr)
-    DoPasswordSubset(*item);
+  if (item) {
+    CallAfter(&PasswordSafeFrame::DoPasswordSubset, item);
+  }
 }
 
 void PasswordSafeFrame::OnPasswordQRCode(wxCommandEvent &evt)
@@ -1065,18 +1111,24 @@ void PasswordSafeFrame::OnPasswordQRCode(wxCommandEvent &evt)
     CItemData rueItem;
     CItemData* item = GetSelectedEntry(evt, rueItem);
     if (item != nullptr) {
-#ifndef NO_QR
-    QRCodeDlg dlg(this, item->GetPassword(),
-              towxstring(CItemData::FieldName(CItem::PASSWORD)) + _T(" of ") +
-              towxstring(item->GetGroup()) +
-              _T('[') + towxstring(item->GetTitle()) + _T(']') +
-              _T(':') + towxstring(item->GetUser()));
-      dlg.ShowModal();
-#endif
+      CallAfter(&PasswordSafeFrame::DoPasswordQRCode, item);
     }
   }
 }
 
+void PasswordSafeFrame::DoPasswordQRCode(CItemData* item)
+{
+#ifndef NO_QR
+  if (item) {
+    ShowModalAndGetResult<QRCodeDlg>(this, item->GetPassword(),
+            towxstring(CItemData::FieldName(CItem::PASSWORD)) + _T(" of ") +
+            towxstring(item->GetGroup()) +
+            _T('[') + towxstring(item->GetTitle()) + _T(']') +
+            _T(':') + towxstring(item->GetUser()));
+  }
+#endif
+}
+
 /*!
  * wxEVT_COMMAND_MENU_SELECTED event handler for ID_PROTECT
  */
