@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -39,10 +39,12 @@ BEGIN_EVENT_TABLE( MergeDlg, wxDialog )
   EVT_BUTTON( ID_ADVANCED,  MergeDlg::OnAdvancedSelection )
 END_EVENT_TABLE()
 
-MergeDlg::MergeDlg(wxWindow* parent, PWScore* core) :
+MergeDlg::MergeDlg(wxWindow *parent, PWScore* core, const wxString& filename) :
                       wxDialog(parent, wxID_ANY, wxString(_("Merge Another Database"))),
                       m_core(core), m_selection(new SelectionCriteria), m_dbPanel(nullptr)
 {
+  wxASSERT(!parent || parent->IsTopLevel());
+
   const wxString filePrompt(wxString(_("Choose Database to Merge into \"")) <<
                                           towxstring(m_core->GetCurFile()) << wxT("\""));
   const wxString filePickerCtrlTitle(_("Please Choose a Database to Merge into current database"));
@@ -50,7 +52,7 @@ MergeDlg::MergeDlg(wxWindow* parent, PWScore* core) :
   wxBoxSizer* dlgSizer = new wxBoxSizer(wxVERTICAL);
 
   //4th arg = true means the panel validates automatically
-  m_dbPanel = new DbSelectionPanel(this, filePrompt, filePickerCtrlTitle, true, core, 2);
+  m_dbPanel = new DbSelectionPanel(this, filePrompt, filePickerCtrlTitle, true, core, 2, wxID_OK, filename);
 
   dlgSizer->Add(m_dbPanel, wxSizerFlags().Expand().Proportion(1).Border());
 
@@ -61,14 +63,21 @@ MergeDlg::MergeDlg(wxWindow* parent, PWScore* core) :
   wxSizer* buttons = CreateStdDialogButtonSizer(wxOK|wxCANCEL|wxHELP);
   //This might not be a very wise thing to do.  We are only supposed to add certain
   //pre-defined button-ids to StdDlgBtnSizer
-  buttons->Add(new wxButton(this, ID_ADVANCED, _("Advanced...")), wxSizerFlags().DoubleBorder(wxLEFT|wxRIGHT));
-  dlgSizer->Add(buttons, wxSizerFlags().Border(wxLEFT|wxRIGHT, SideMargin).Expand());
-
+  auto advancedButton = new wxButton(this, ID_ADVANCED, _("&Advanced..."), wxDefaultPosition, wxDefaultSize, 0);
+  buttons->Add(advancedButton, 0, wxLEFT|wxRIGHT|wxALIGN_CENTER_VERTICAL, SideMargin);
+  
+  dlgSizer->Add(buttons, 0, wxLEFT|wxRIGHT|wxEXPAND, SideMargin);
+  
   dlgSizer->AddSpacer(BottomMargin);
 
   SetSizerAndFit(dlgSizer);
 }
 
+MergeDlg* MergeDlg::Create(wxWindow *parent, PWScore* core, const wxString& filename)
+{
+  return new MergeDlg(parent, core, filename);
+}
+
 MergeDlg::~MergeDlg()
 {
   delete m_selection;
@@ -111,8 +120,12 @@ IMPLEMENT_CLASS_TEMPLATE( AdvancedSelectionDlg, wxDialog, AdvancedMergeOptions )
 
 void MergeDlg::OnAdvancedSelection(wxCommandEvent& )
 {
-  AdvancedSelectionDlg<AdvancedMergeOptions> dlg(this, m_selection);
-  dlg.ShowModal();
+  CallAfter(&MergeDlg::DoAdvancedSelection);
+}
+
+void MergeDlg::DoAdvancedSelection()
+{
+  ShowModalAndGetResult<AdvancedSelectionDlg<AdvancedMergeOptions>>(this, m_selection);
 }
 
 wxString MergeDlg::GetOtherSafePath() const
