@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -26,19 +26,21 @@ class MergeDlg : public wxDialog
   DECLARE_EVENT_TABLE()
   
 public:
-  MergeDlg(wxWindow* parent, PWScore* core);
+  static MergeDlg* Create(wxWindow *parent, PWScore* core, const wxString &filename = wxEmptyString);
   ~MergeDlg();
-
+protected:
+  MergeDlg(wxWindow *parent, PWScore* core, const wxString &filename);
   void OnAdvancedSelection( wxCommandEvent& evt );
-
+public:
   wxString GetOtherSafePath() const;
   StringX GetOtherSafeCombination() const;
   SelectionCriteria GetSelectionCriteria() const;
   
 private:
-  PWScore* m_core;
-  SelectionCriteria* m_selection;
-  DbSelectionPanel* m_dbPanel;
+  void DoAdvancedSelection();
+  PWScore* m_core = nullptr;
+  SelectionCriteria* m_selection = nullptr;
+  DbSelectionPanel* m_dbPanel = nullptr;
 };
 
 #endif // _MERGEDLG_H_
