@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -144,25 +144,20 @@ const wxString DCAStrings[] = {
 /*!
  * OptionsPropertySheetDlg constructors
  */
-
-OptionsPropertySheetDlg::OptionsPropertySheetDlg(PWScore &core) : m_core(core)
-{
-  Init();
-}
-
-OptionsPropertySheetDlg::OptionsPropertySheetDlg( wxWindow* parent, PWScore &core, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
+OptionsPropertySheetDlg::OptionsPropertySheetDlg(wxWindow *parent, PWScore &core, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
   : m_core(core)
 {
-  Init();
-  Create(parent, id, caption, pos, size, style);
-}
+  const wxSize imageSize(64, 64);
 
-/*!
- * OptionsPropertySheetDlg creator
- */
+  m_ImageList = new wxImageList(imageSize.GetWidth(), imageSize.GetHeight());
+  m_ImageList->Add(wxIcon(backups_xpm));        // https://www.pngrepo.com/svg/224774/database
+  m_ImageList->Add(wxIcon(display_xpm));        // https://www.pngrepo.com/svg/131986/computer
+  m_ImageList->Add(wxIcon(miscellaneous_xpm));  // https://www.pngrepo.com/svg/230756/test
+  m_ImageList->Add(wxIcon(history_xpm));        // https://www.pngrepo.com/svg/95031/file
+  m_ImageList->Add(wxIcon(security_xpm));       // https://www.pngrepo.com/svg/219291/security-shield
+  m_ImageList->Add(wxIcon(shortcuts_xpm));      // https://www.pngrepo.com/svg/153991/keyboard
+  m_ImageList->Add(wxIcon(system_xpm));         // https://www.pngrepo.com/svg/230728/settings-gear
 
-bool OptionsPropertySheetDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
-{
 ////@begin OptionsPropertySheetDlg creation
   SetSheetStyle(wxPROPSHEET_LISTBOOK);
   SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY|wxWS_EX_BLOCK_EVENTS);
@@ -176,7 +171,11 @@ bool OptionsPropertySheetDlg::Create( wxWindow* parent, wxWindowID id, const wxS
 ////@end OptionsPropertySheetDlg creation
   wxCommandEvent dummyEv;
   OnSuffixCBSet(dummyEv);
-  return true;
+}
+
+OptionsPropertySheetDlg* OptionsPropertySheetDlg::Create(wxWindow *parent, PWScore &core, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style)
+{
+  return new OptionsPropertySheetDlg(parent, core, id, caption, pos, size, style);
 }
 
 /*!
@@ -190,72 +189,6 @@ OptionsPropertySheetDlg::~OptionsPropertySheetDlg()
   delete m_ImageList;
 }
 
-/*!
- * Member initialisation
- */
-
-void OptionsPropertySheetDlg::Init()
-{
-////@begin OptionsPropertySheetDlg member initialisation
-  m_Backups_Panel = nullptr;
-  m_Backups_DefaultPrefixRB = nullptr;
-  m_Backups_UserPrefixRB = nullptr;
-  m_Backups_UserPrefixTXT = nullptr;
-  m_Backups_SuffixCB = nullptr;
-  m_Backups_MaxIncrSB = nullptr;
-  m_Backups_SuffixExampleST = nullptr;
-  m_Backups_DefaultDirRB = nullptr;
-  m_Backups_UserDirRB = nullptr;
-  m_Backups_UserDirTXT = nullptr;
-  m_Backups_DirBN = nullptr;
-
-  m_Display_Panel = nullptr;
-  m_Display_ShowPasswordInTreeCB = nullptr;
-  m_Display_PreExpiryWarnCB = nullptr;
-  m_Display_PreExpiryWarnDaysSB = nullptr;
-
-  m_Misc_Panel = nullptr;
-  m_Misc_DoubleClickActionCB = nullptr;
-  m_Misc_ShiftDoubleClickActionCB = nullptr;
-  m_Misc_DefaultUsernameTXT = nullptr;
-  m_Misc_DefaultUsernameLBL = nullptr;
-
-  m_PasswordHistory_Panel = nullptr;
-  m_PasswordHistory_SaveCB = nullptr;
-  m_PasswordHistory_NumDefaultSB = nullptr;
-  m_PasswordHistory_DefaultExpiryDaysSB = nullptr;
-  m_PasswordHistory_ApplyBN = nullptr;
-  m_PasswordHistory_NoChangeRB = nullptr;
-  m_PasswordHistory_StopRB = nullptr;
-  m_PasswordHistory_StartRB = nullptr;
-  m_PasswordHistory_SetMaxRB = nullptr;
-  m_PasswordHistory_ClearRB = nullptr;
-  m_PasswordHistory_Apply2ProtectedCB = nullptr;
-
-  m_Security_Panel = nullptr;
-  m_Security_LockOnIdleTimeoutCB = nullptr;
-  m_Security_IdleTimeoutSB = nullptr;
-
-  m_Shortcuts_Panel = nullptr;
-
-  m_System_Panel = nullptr;
-  m_System_UseSystemTrayCB = nullptr;
-  m_System_MaxREItemsSB = nullptr;
-  m_System_SystemTrayWarningST = nullptr;
-////@end OptionsPropertySheetDlg member initialisation
-
-  const wxSize imageSize(64, 64);
-
-  m_ImageList = new wxImageList(imageSize.GetWidth(), imageSize.GetHeight());
-  m_ImageList->Add(wxIcon(backups_xpm));        // https://www.pngrepo.com/svg/224774/database
-  m_ImageList->Add(wxIcon(display_xpm));        // https://www.pngrepo.com/svg/131986/computer
-  m_ImageList->Add(wxIcon(miscellaneous_xpm));  // https://www.pngrepo.com/svg/230756/test
-  m_ImageList->Add(wxIcon(history_xpm));        // https://www.pngrepo.com/svg/95031/file
-  m_ImageList->Add(wxIcon(security_xpm));       // https://www.pngrepo.com/svg/219291/security-shield
-  m_ImageList->Add(wxIcon(shortcuts_xpm));      // https://www.pngrepo.com/svg/153991/keyboard
-  m_ImageList->Add(wxIcon(system_xpm));         // https://www.pngrepo.com/svg/230728/settings-gear
-}
-
 /*!
  * Control creation for OptionsPropertySheetDlg
  */
@@ -531,6 +464,22 @@ wxPanel* OptionsPropertySheetDlg::CreateDisplayPanel(const wxString& title)
 
   itemBoxSizer39->Add(m_Display_PreExpiryWarnDaysSB, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
+  wxCheckBox* display_ShowMenuSeparatorCB = new wxCheckBox( itemPanel29, ID_CHECKBOX43, _("Show Menu Separator"), wxDefaultPosition, wxDefaultSize, 0 );
+  display_ShowMenuSeparatorCB->SetValue(false);
+  itemBoxSizer30->Add(display_ShowMenuSeparatorCB, 0, wxALIGN_LEFT|wxALL, 5);
+
+  wxCheckBox* display_AutoAdjColWidthCB = new wxCheckBox( itemPanel29, ID_CHECKBOX44, _("Auto-adjust column width"), wxDefaultPosition, wxDefaultSize, 0 );
+  display_AutoAdjColWidthCB->SetValue(false);
+  itemBoxSizer30->Add(display_AutoAdjColWidthCB, 0, wxALIGN_LEFT|wxALL, 5);
+  
+  wxCheckBox* display_ToolbarShowTextCB = new wxCheckBox( itemPanel29, ID_CHECKBOX45, _("Show Text in Toolbar"), wxDefaultPosition, wxDefaultSize, 0 );
+  display_ToolbarShowTextCB->SetValue(false);
+  itemBoxSizer30->Add(display_ToolbarShowTextCB, 0, wxALIGN_LEFT|wxALL, 5);
+  
+  wxCheckBox* display_ToolbarShowAliasSelectionCB = new wxCheckBox( itemPanel29, ID_CHECKBOX46, _("Show Alias Selection in Add && Edit"), wxDefaultPosition, wxDefaultSize, 0 );
+  display_ToolbarShowAliasSelectionCB->SetValue(false);
+  itemBoxSizer30->Add(display_ToolbarShowAliasSelectionCB, 0, wxALIGN_LEFT|wxALL, 5);
+  
   wxStaticText* itemStaticText42 = new wxStaticText( itemPanel29, wxID_STATIC, _("days before passwords expire"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer39->Add(itemStaticText42, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -552,7 +501,11 @@ wxPanel* OptionsPropertySheetDlg::CreateDisplayPanel(const wxString& title)
   display_GroupsFirstCB->SetValidator( wxGenericValidator(& m_Display_GroupsFirst) );
   display_PreExpiryWarnCB->SetValidator( wxGenericValidator(& m_Display_PreExpiryWarn) );
   display_TreeDisplayStatusAtOpenRB->SetValidator( wxGenericValidator(& m_Display_TreeDisplayStatusAtOpen) );
-
+  display_ShowMenuSeparatorCB->SetValidator( wxGenericValidator(& m_Display_ShowMenuSeparator) );
+  display_AutoAdjColWidthCB->SetValidator( wxGenericValidator(& m_Display_AutoAdjColWidth) );
+  display_ToolbarShowTextCB->SetValidator( wxGenericValidator(& m_Display_ToolbarShowText) );
+  display_ToolbarShowAliasSelectionCB->SetValidator( wxGenericValidator(& m_Display_AliasSelection) );
+  
   return itemPanel29;
 }
 
@@ -1034,7 +987,11 @@ void OptionsPropertySheetDlg::PrefsToPropSheet()
   m_Display_PreExpiryWarnDaysSB->SetValue(prefs->GetPref(PWSprefs::PreExpiryWarnDays));
   m_Display_PreExpiryWarnDaysSB->Enable(m_Display_PreExpiryWarn);
   m_Display_TreeDisplayStatusAtOpen = prefs->GetPref(PWSprefs::TreeDisplayStatusAtOpen);
-
+  m_Display_ShowMenuSeparator = prefs->GetPref(PWSprefs::ShowMenuSeparator);
+  m_Display_AutoAdjColWidth = prefs->GetPref(PWSprefs::AutoAdjColWidth);
+  m_Display_ToolbarShowText = prefs->GetPref(PWSprefs::ToolbarShowText);
+  m_Display_AliasSelection = prefs->GetPref(PWSprefs::ShowAliasSelection);
+  
   // Misc. preferences
   m_Misc_ConfirmDelete = !prefs->GetPref(PWSprefs::DeleteQuestion);
   m_Misc_MaintainDatetimeStamps = prefs->GetPref(PWSprefs::MaintainDateTimeStamps);
@@ -1212,6 +1169,10 @@ void OptionsPropertySheetDlg::PropSheetToPrefs()
   prefs->SetPref(PWSprefs::ShowPasswordInTree, m_Display_ShowPasswordInTreeCB->GetValue(), true);
 
   prefs->SetPref(PWSprefs::TreeDisplayStatusAtOpen, m_Display_TreeDisplayStatusAtOpen, true);
+  prefs->SetPref(PWSprefs::ShowMenuSeparator, m_Display_ShowMenuSeparator);
+  prefs->SetPref(PWSprefs::AutoAdjColWidth, m_Display_AutoAdjColWidth);
+  prefs->SetPref(PWSprefs::ToolbarShowText, m_Display_ToolbarShowText);
+  prefs->SetPref(PWSprefs::ShowAliasSelection, m_Display_AliasSelection);
   prefs->SetPref(PWSprefs::ShowNotesDefault, m_Display_ShowNotesInEdit, true);
   prefs->SetPref(PWSprefs::MaintainDateTimeStamps, m_Misc_MaintainDatetimeStamps, true);
   prefs->SetPref(PWSprefs::UseDefaultUser, m_Misc_UseDefUsername, true);
