@@ -671,11 +671,7 @@ void pwFiltersFTChoiceEditor::BeginEdit(int row, int col, wxGrid* grid)
   
   wxGridCellEditorEvtHandler* evtHandler = NULL;
   if(m_control) {
-#if wxVERSION_NUMBER > 3005
-    evtHandler = wxDynamicCast(m_control->GetEventHandler(), wxGridCellEditorEvtHandler);
-#else
     evtHandler = static_cast<wxGridCellEditorEvtHandler *>(m_control->GetEventHandler());
-#endif
   }
 
   // Don't immediately end if we get a kill focus event within BeginEdit
@@ -972,11 +968,7 @@ void pwFiltersLCChoiceEditor::BeginEdit(int row, int col, wxGrid* grid)
 
   wxGridCellEditorEvtHandler* evtHandler = NULL;
   if(m_control) {
-#if wxVERSION_NUMBER > 3005
-    evtHandler = wxDynamicCast(m_control->GetEventHandler(), wxGridCellEditorEvtHandler);
-#else
     evtHandler = static_cast<wxGridCellEditorEvtHandler *>(m_control->GetEventHandler());
-#endif
   }
   
   // Don't immediately end if we get a kill focus event within BeginEdit
