@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -40,11 +40,13 @@
 
 #include "os/cleanup.h"
 #include "os/dir.h"
+#include "os/file.h"
 #include "core/PWSfile.h"
 #include "core/PWSLog.h"
 #include "core/PWSprefs.h"
 #include "core/PWSrand.h"
 #include "core/SysInfo.h"
+#include "core/PWSdirs.h"
 #include "wxUtilities.h"
 
 #if defined(__X__) || defined(__WXGTK__) || defined(__WXOSX__)
@@ -212,6 +214,7 @@ void PWSafeApp::Init()
   wxLocale::AddCatalogLookupPathPrefix(L"/usr/local");
 #if defined(__WXDEBUG__) || defined(_DEBUG) || defined(DEBUG)
   wxLocale::AddCatalogLookupPathPrefix(L"../I18N/mos");
+  wxLocale::AddCatalogLookupPathPrefix(L"src/ui/wxWidgets/I18N/mos"); // located in cmake's build directory
 #endif
 
 ////@begin PWSafeApp member initialisation
@@ -271,8 +274,30 @@ bool PWSafeApp::OnInit()
   // Get the locale environment variable 'LC_CTYPE' specified by the environment
   // For instance, the behavior of function 'wcstombs' depends on the LC_CTYPE 
   // category of the selected C locale.
-  setlocale(LC_CTYPE, "");
+#if defined(__WXMAC__)
+  const char *lcCType = setlocale(LC_CTYPE, NULL);
 
+  if (lcCType && strcmp(lcCType, "UTF-8")) { // MAC OS only have UTF-8 as default, but conversion with this string is not running well
+    setlocale(LC_CTYPE, "");
+  }
+  else {
+    const char *env_lc_cType = std::getenv("LC_CTYPE");
+    
+    if(env_lc_cType) {
+      setlocale(LC_CTYPE, env_lc_cType);
+    }
+    else {
+      setlocale(LC_CTYPE, "en_US.UTF-8");
+    }
+  }
+  // This value must be set for mac OS starting with version 11, but is no problem for earlier versions, see
+  // https://trac.wxwidgets.org/ticket/19023
+  setlocale(LC_NUMERIC, "C");
+#else
+  setlocale(LC_CTYPE, "");
+#endif
+  setlocale(LC_TIME, "");
+  
   //Used by help subsystem
   wxFileSystem::AddHandler(new wxArchiveFSHandler);
 
@@ -453,17 +478,15 @@ bool PWSafeApp::OnInit()
       _("Password Safe: Error initializing help"), wxOK | wxICON_ERROR);
 #endif
     }
-    
   }
 
   if (!cmd_closed && !cmd_silent && !cmd_minimized) {
     // Get the file, r/w mode and password from user
     // Note that file may be new
-    SafeCombinationEntryDlg* initWindow = new SafeCombinationEntryDlg(nullptr, m_core);
+    DestroyWrapper<SafeCombinationEntryDlg> initWindowWrapper(nullptr, m_core);
+    SafeCombinationEntryDlg *initWindow = initWindowWrapper.Get();
     int returnValue = initWindow->ShowModal();
 
-    initWindow->Destroy();
-
     if (returnValue != wxID_OK) {
       return false;
     }
@@ -477,6 +500,30 @@ bool PWSafeApp::OnInit()
   }
 
   RestoreFrameCoords();
+
+  // Load the default filter, if any
+  PWSprefs::ConfigOption configoption;
+  std::wstring wsCnfgFile = PWSprefs::GetConfigFile(configoption);
+  std::wstring wsAutoLoad;
+  if (configoption == PWSprefs::CF_NONE ||
+      configoption == PWSprefs::CF_REGISTRY) {
+     // Need to use configuration directory instead
+    wsAutoLoad = PWSdirs::GetConfigDir() + L"/" + L"autoload_filters.xml";
+  } else {
+    std::wstring wsCnfgDrive, wsCnfgDir, wsCnfgFileName, wsCnfgExt;
+    pws_os::splitpath(wsCnfgFile, wsCnfgDrive, wsCnfgDir, wsCnfgFileName, wsCnfgExt);
+    wsAutoLoad = pws_os::makepath(wsCnfgDrive, wsCnfgDir, L"autoload_filters", L".xml");
+  }
+  
+  if (pws_os::FileExists(wsAutoLoad)) {
+    stringT strErrors = L"";
+    int rc = m_frame->ImportFilterXMLFile(FPOOL_AUTOLOAD, L"", wsAutoLoad,
+                                          L"", strErrors, &anAsker, nullptr); // Only summary will be reported
+    if (rc != PWScore::SUCCESS) {
+      wxMessageBox(towxstring(strErrors), _("Unable to import \"autoload_filters.xml\""), wxOK | wxICON_ERROR);
+    }
+  }
+
   if (!cmd_silent)
     m_frame->Show();
   if (cmd_minimized)
@@ -500,6 +547,8 @@ bool PWSafeApp::OnInit()
   }
   if (PWSprefs::GetInstance()->GetPref(PWSprefs::UseSystemTray))
     m_frame->ShowTrayIcon();
+
+  m_frame->Register(); // register modal dialog hook
   return true;
 }
 
@@ -594,6 +643,7 @@ bool PWSafeApp::ActivateLanguage(wxLanguage language, bool tryOnly)
     if ( !translations->AddCatalog(DOMAIN_) ) {
       pws_os::Trace(L"Couldn't load %ls language catalog for %ls\n", ToStr(DOMAIN_), ToStr(wxLocale::GetLanguageName(language)));
       translations->SetLanguage(wxLANGUAGE_ENGLISH);
+      language = wxLANGUAGE_ENGLISH; // Back to default language english
     }
     bRes = translations->IsLoaded(DOMAIN_);
   }
@@ -605,6 +655,19 @@ bool PWSafeApp::ActivateLanguage(wxLanguage language, bool tryOnly)
     // (re)set global translation and take care of occupied memory by wxTranslations
     wxTranslations::Set(translations);
     isHelpActivated = ActivateHelp(language);
+
+    const wxLanguageInfo *langInfo = nullptr;
+    langInfo = wxLocale::GetLanguageInfo(language);
+    if(langInfo) {
+      wxString envString = langInfo->CanonicalName + ".UTF-8";
+      setlocale(LC_CTYPE, envString.c_str());
+      setlocale(LC_TIME, envString.c_str());
+#if defined(__WXMAC__)
+      // This value must be set for mac OS starting with version 11, but is no problem for earlier versions, see
+      // https://trac.wxwidgets.org/ticket/19023
+      setlocale(LC_NUMERIC, "C");
+#endif
+    }
   }
   return bRes;
 }
@@ -618,8 +681,11 @@ int PWSafeApp::OnExit()
   m_idleTimer->Stop();
   recentDatabases().Save();
   PWSprefs *prefs = PWSprefs::GetInstance();
-  if (m_core.IsDbOpen())
+  if (m_core.IsDbOpen()) {
     prefs->SetPref(PWSprefs::CurrentFile, m_core.GetCurFile());
+    // Don't leave dangling locks!
+    m_core.SafeUnlockCurFile();
+  }
   // Save Application related preferences
   prefs->SaveApplicationPreferences();
   // Save shortcuts, if changed
@@ -658,8 +724,11 @@ void PWSafeApp::ConfigureIdleTimer()
 void PWSafeApp::OnIdleTimer(wxTimerEvent &evt)
 {
   if (evt.GetId() == IDLE_TIMER_ID && PWSprefs::GetInstance()->GetPref(PWSprefs::LockDBOnIdleTimeout)) {
-    if (m_frame != nullptr && !m_frame->GetCurrentSafe().IsEmpty()) {
+    if (m_frame != nullptr && !m_frame->IsClosed() && !m_frame->IsLocked()) {
+      m_idleTimer->Stop(); // Stop as with modal dialog a dialog will stay open
       m_frame->IconizeOrHideAndLock();
+      if(!m_frame->IsLocked()) // If not locked, start timer again
+        ConfigureIdleTimer();
     }
   }
 }
@@ -701,7 +770,7 @@ void PWSafeApp::RestoreFrameCoords(void)
   long top, bottom, left, right;
   PWSprefs::GetInstance()->GetPrefRect(top, bottom, left, right);
   if (!(left == -1 && top == -1 && right == -1 && bottom == -1)) {
-    wxRect rcApp(left, top, right - left, bottom - top);
+    wxRect rcApp(static_cast<int>(left), static_cast<int>(top), static_cast<int>(right - left), static_cast<int>(bottom - top));
 
     int displayWidth, displayHeight;
     ::wxDisplaySize(&displayWidth, &displayHeight);
@@ -831,7 +900,7 @@ void PWSafeApp::OnHelp(wxCommandEvent& evt)
     StringToStringMap& helpmap = GetHelpMap();
     StringToStringMap::iterator itr = helpmap.find(keyName);
     if (itr != helpmap.end()) {
-      wxHtmlModalHelp help(wxGetApp().GetTopWindow(), helpFileNamePath, itr->second, wxHF_DEFAULT_STYLE);
+      wxHtmlModalHelp help(window, helpFileNamePath, itr->second, wxHF_DEFAULT_STYLE);
     }
     else {
 #ifdef __WXDEBUG__
@@ -876,3 +945,7 @@ void PWSafeApp::RestartIdleTimer()
     m_idleTimer->Start(interval, wxTIMER_CONTINUOUS);
   }
 }
+
+bool PWSafeApp::IsCloseInProgress() const {
+  return m_frame->IsCloseInProgress();
+}
