@@ -2,7 +2,7 @@
  * Initial version created by Rony Shapiro 
  * on Wed 14 Jan 2009 10:11:39 PM IST.
  * 
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -98,6 +98,7 @@ public:
 
   wxIconBundle GetAppIcons() const { return m_appIcons; }
 
+  bool IsCloseInProgress() const;
 private:
   PWScore m_core;
   wxTimer *m_idleTimer;
@@ -123,6 +124,7 @@ public:
   bool ActivateLanguage(wxLanguage language, bool tryOnly);
   wxLanguage GetSystemLanguage();
   wxLanguage GetSelectedLanguage();
+  PasswordSafeFrame* GetPasswordSafeFrame() { return m_frame; }
 };
 
 /*!
