@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -56,6 +56,7 @@ BEGIN_EVENT_TABLE( PasswordPolicyDlg, wxDialog )
   EVT_BUTTON(   wxID_OK             , PasswordPolicyDlg::OnOkClick              )
   EVT_BUTTON(   wxID_CANCEL         , PasswordPolicyDlg::OnCancelClick          )
   EVT_BUTTON(   wxID_HELP           , PasswordPolicyDlg::OnHelpClick            )
+  EVT_CLOSE( PasswordPolicyDlg::OnClose )
 ////@end PasswordPolicyDlg event table entries
 
 END_EVENT_TABLE()
@@ -64,22 +65,19 @@ END_EVENT_TABLE()
  * PasswordPolicyDlg constructor
  */
 
-PasswordPolicyDlg::PasswordPolicyDlg( wxWindow* parent, PWScore &core,
+PasswordPolicyDlg::PasswordPolicyDlg(wxWindow *parent, PWScore &core,
                                   const PSWDPolicyMap &polmap, DialogType type,
                                   wxWindowID id, const wxString& caption,
                                   const wxPoint& pos, const wxSize& size, long style )
 : m_core(core), m_MapPSWDPLC(polmap), m_DialogType(type)
 {
-  Init();
-  Create(parent, type, id, caption, pos, size, style);
-}
+  wxASSERT(!parent || parent->IsTopLevel());
 
-/*!
- * PasswordPolicyDlg creator
- */
-
-bool PasswordPolicyDlg::Create( wxWindow* parent, DialogType type, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
-{
+  // Collect all policy names to display in combobox control
+  for (auto& policy : m_MapPSWDPLC) {
+    m_Policynames.Add(stringx2std(policy.first));
+  }
+  
 ////@begin PasswordPolicyDlg creation
   m_DialogType = type;
   SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
@@ -92,7 +90,14 @@ bool PasswordPolicyDlg::Create( wxWindow* parent, DialogType type, wxWindowID id
   }
   Centre();
 ////@end PasswordPolicyDlg creation
-  return true;
+}
+
+PasswordPolicyDlg* PasswordPolicyDlg::Create(wxWindow *parent, PWScore &core,
+                                  const PSWDPolicyMap &polmap, DialogType type,
+                                  wxWindowID id, const wxString& caption,
+                                  const wxPoint& pos, const wxSize& size, long style)
+{
+  return new PasswordPolicyDlg(parent, core, polmap, type, id, caption, pos, size, style);
 }
 
 void PasswordPolicyDlg::SetDefaultSymbolDisplay(bool restore_defaults)
@@ -111,54 +116,6 @@ void PasswordPolicyDlg::SetDefaultSymbolDisplay(bool restore_defaults)
   m_OwnSymbols->SetValue(m_Symbols);
 }
 
-/*!
- * PasswordPolicyDlg destructor
- */
-
-PasswordPolicyDlg::~PasswordPolicyDlg()
-{
-////@begin PasswordPolicyDlg destruction
-////@end PasswordPolicyDlg destruction
-}
-
-/*!
- * Member initialisation
- */
-
-void PasswordPolicyDlg::Init()
-{
-////@begin PasswordPolicyDlg member initialisation
-  m_pwpLenCtrl = nullptr;
-  m_pwMinsGSzr = nullptr;
-  m_pwpUseLowerCtrl = nullptr;
-  m_pwNumLCbox = nullptr;
-  m_pwpLCSpin = nullptr;
-  m_pwpUseUpperCtrl = nullptr;
-  m_pwNumUCbox = nullptr;
-  m_pwpUCSpin = nullptr;
-  m_pwpUseDigitsCtrl = nullptr;
-  m_pwNumDigbox = nullptr;
-  m_pwpDigSpin = nullptr;
-  m_pwpSymCtrl = nullptr;
-  m_pwNumSymbox = nullptr;
-  m_pwpSymSpin = nullptr;
-  m_OwnSymbols = nullptr;
-  m_pwpEasyCtrl = nullptr;
-  m_pwpPronounceCtrl = nullptr;
-  m_pwpHexCtrl = nullptr;
-
-  m_UseDatabasePolicyCtrl = nullptr;
-  m_PoliciesSelectionCtrl = nullptr;
-  m_passwordCtrl = nullptr;
-  m_itemStaticBoxSizer6 = nullptr;
-////@end PasswordPolicyDlg member initialisation
-
-  // Collect all policy names to display in combobox control
-  for (auto& policy : m_MapPSWDPLC) {
-    m_Policynames.Add(stringx2std(policy.first));
-  }
-}
-
 /*!
  * Control creation for PasswordPolicyDlg
  */
@@ -595,18 +552,6 @@ void PasswordPolicyDlg::OnOkClick( wxCommandEvent& )
   EndModal(wxID_OK);
 }
 
-/*!
- * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL
- */
-
-void PasswordPolicyDlg::OnCancelClick( wxCommandEvent& event )
-{
-////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL in PasswordPolicyDlg.
-  // Before editing this code, remove the block markers.
-  event.Skip();
-////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL in PasswordPolicyDlg.
-}
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
  */
@@ -659,7 +604,8 @@ void PasswordPolicyDlg::SetPolicyData(const wxString &policyname, const PWPolicy
       m_PoliciesSelectionCtrl->SetSelection(index);
     }
     else {
-      m_PoliciesSelectionCtrl->SetSelection(0);
+      if(m_PoliciesSelectionCtrl->GetCount()) // When entry is in the list
+        m_PoliciesSelectionCtrl->SetSelection(0);
     }
   }
 }
@@ -853,9 +799,9 @@ void PasswordPolicyDlg::OnPolicynameSelection( wxCommandEvent& WXUNUSED(event) )
 
 void PasswordPolicyDlg::OnGeneratePassword( wxCommandEvent& WXUNUSED(event) )
 {
-  UpdatePolicy();
-
-  m_passwordCtrl->SetValue(m_st_pp.MakeRandomPassword().c_str());
+  if (UpdatePolicy()) {
+    m_passwordCtrl->SetValue(m_st_pp.MakeRandomPassword().c_str());
+  }
 }
 
 /*!
@@ -900,3 +846,59 @@ void PasswordPolicyDlg::OnAtLeastPasswordChars( wxSpinEvent& WXUNUSED(event) )
     m_pwpLenCtrl->SetValue(total);
   }
 }
+
+bool PasswordPolicyDlg::SyncAndQueryCancel(bool showDialog) {
+  // no need to check when used in R/O or generator mode
+  if (m_DialogType == DialogType::GENERATOR || m_core.IsReadOnly()) {
+    return true;
+  }
+  return QueryCancelDlg::SyncAndQueryCancel(showDialog);
+}
+
+uint32_t PasswordPolicyDlg::GetChanges() const {
+  uint32_t changes = Changes::None;
+
+  if (m_polname != m_oldpolname) {
+    changes |= Changes::Name;
+  }
+
+  if (m_pwUseLowercase != m_oldpwUseLowercase
+      || m_pwUseUppercase != m_oldpwUseUppercase
+      || m_pwUseDigits != m_oldpwUseDigits
+      || m_pwUseSymbols != m_oldpwUseSymbols
+      || m_pwUseHex != m_oldpwUseHex
+      || m_pwUseEasyVision != m_oldpwUseEasyVision
+      || m_pwMakePronounceable != m_oldpwMakePronounceable) {
+    changes |= Changes::Flags;
+  }
+
+  if (m_pwdefaultlength != m_oldpwdefaultlength) {
+    changes |= Changes::Length;
+  }
+
+  if (m_pwLowerMinLength != m_oldpwLowerMinLength) {
+    changes |= Changes::LowerMinLength;
+  }
+
+  if (m_pwUpperMinLength != m_oldpwUpperMinLength) {
+    changes |= Changes::UpperMinLength;
+  }
+
+  if (m_pwDigitMinLength != m_oldpwDigitMinLength) {
+    changes |= Changes::DigitMinLength;
+  }
+
+  if (m_pwSymbolMinLength != m_oldpwSymbolMinLength) {
+    changes |= Changes::SymbolMinLength;
+  }
+
+  if (m_Symbols != m_oldSymbols) {
+    changes |= Changes::Symbols;
+  }
+
+  return changes;
+}
+
+bool PasswordPolicyDlg::IsChanged() const {
+  return GetChanges() != Changes::None;
+}
