@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -18,12 +18,13 @@
  */
 
 ////@begin includes
-#include <wx/event.h>
-#include <wx/toolbar.h>
+#include <wx/aui/auibar.h>
 
 #include "core/ItemData.h"
 ////@end includes
 
+#define SEARCHBAR_STYLE wxAUI_TB_DEFAULT_STYLE|wxAUI_TB_GRIPPER|wxAUI_TB_PLAIN_BACKGROUND
+
 ////@begin forward declarations
 class PasswordSafeFrame;
 struct SelectionCriteria;
@@ -65,6 +66,13 @@ public:
     if (restart) { InitIndex(); }
     PrintLabel();
   }
+  
+  const pws_os::CUUID& GetUUID(size_t idx) {
+    if(idx < m_indices.size()) {
+      return m_indices[idx];
+    }
+    return pws_os::CUUID::NullUUID();
+  }
 
   SearchPointer& operator++();
   SearchPointer& operator--();
@@ -79,16 +87,10 @@ private:
  * PasswordSafeSearch class declaration
  */
 
-class PasswordSafeSearch : public wxEvtHandler
+class PasswordSafeSearch : public wxAuiToolBar
 {
-  DECLARE_CLASS( PasswordSafeSearch )
-
-  DECLARE_NO_COPY_CLASS(PasswordSafeSearch)
-
 public:
-  /// Constructors
-  PasswordSafeSearch(PasswordSafeFrame* parent);
-  /// Destructor
+  PasswordSafeSearch(wxWindow *parent, wxWindowID id=wxID_ANY, const wxPoint &position=wxDefaultPosition, const wxSize &size=wxDefaultSize, long style=SEARCHBAR_STYLE);
   ~PasswordSafeSearch();
 
   void OnSearchClose(wxCommandEvent& event);
@@ -96,9 +98,11 @@ public:
   void FindPrevious();
 
   void Activate();
-  void RefreshButtons();
+  void UpdateBitmaps();
+  bool HasTools() const { return GetToolCount() > 0; }
   void Invalidate() { m_searchPointer.Clear(); }
   void ReCreateSearchBar();
+  void SetFocusIntoEditField();
 
 private:
   template <class Iter, class Accessor>
@@ -113,22 +117,29 @@ private:
   void UpdateView();
   void OnSearchTextChanged(wxCommandEvent& event);
   void OnSearchBarTextChar(wxKeyEvent& event);
+  void OnToolBarFindReport(wxCommandEvent& event);
 
-  void CreateSearchBar();
+  bool CreateSearchBar();
   void HideSearchToolbar();
   void ClearToolbarStatusArea();
   void CalculateToolsWidth();
   wxSize CalculateSearchWidth();
   void UpdateStatusAreaWidth();
+  void DoToolBarFindReport(int controlId);
+  
+  void GetAdvancedSearchOptions(int cointrolId);
 
+  void SetModified(bool modified) { m_modified = modified; }
+  bool IsModified() const { return m_modified; }
+  
   template <class Iter, class Accessor>
   void OnDoSearchT( Iter begin, Iter end, Accessor afn);
 
-  wxToolBar*           m_toolbar;
   PasswordSafeFrame*   m_parentFrame;
   SelectionCriteria*   m_criteria;
   SearchPointer        m_searchPointer;
   size_t               m_ToolsWidth;
+  bool                 m_modified;
 };
 
 #endif // _PASSWORDSAFESEARCH_H_
