@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -39,8 +39,7 @@ BEGIN_EVENT_TABLE(QRCodeDlg, wxDialog)
   EVT_TIMER(wxID_ANY, QRCodeDlg::OnTimer)
 END_EVENT_TABLE()
 
-QRCodeDlg::QRCodeDlg(wxWindow *parent,
-                     const StringX &data,
+QRCodeDlg::QRCodeDlg(wxWindow *parent, const StringX &data,
                      const wxString &dlgTitle,
                      const int seconds,
                      const wxPoint &pos,
@@ -48,9 +47,18 @@ QRCodeDlg::QRCodeDlg(wxWindow *parent,
                      long style,
                      const wxString &name) : wxDialog(parent, wxID_ANY, dlgTitle, pos, size, style, name), timer(this), secondsRemaining(seconds)
 {
+  wxASSERT(!parent || parent->IsTopLevel());
+
   CreateControls(data);
 }
 
+QRCodeDlg* QRCodeDlg::Create(wxWindow *parent, const StringX &data, const wxString &dlgTitle,
+                     const int seconds, const wxPoint &pos,
+                     const wxSize &size, long style, const wxString &name)
+{
+  return new QRCodeDlg(parent, data, dlgTitle, seconds, pos, size, style, name);
+}
+
 void QRCodeDlg::CreateControls(const StringX &data)
 {
   wxSizer *dlgSizer = new wxBoxSizer(wxVERTICAL);
@@ -148,8 +156,7 @@ public:
     }
     else
     {
-      QRCodeDlg dlg(nullptr, StringX(argv[1]), "Scan this QR Code and comapre with the program argument");
-      dlg.ShowModal();
+      ShowModalAndGetResult<QRCodeDlg>(this, StringX(argv[1]), "Scan this QR Code and comapre with the program argument");
       // Normall we return true here
       return false;
     }
