@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -51,6 +51,7 @@ public:
   virtual bool TransferFromWindow();
 
   void AllowEmptyCombinationOnce() {m_allowBlank = true;}
+  void ForbidEmptyCombinationOnce() {m_allowBlank = false;}
 
 private:
   StringX* m_str;
@@ -149,6 +150,13 @@ void SafeCombinationCtrl::AllowEmptyCombinationOnce()
     scValidator->AllowEmptyCombinationOnce();
 }
 
+void SafeCombinationCtrl::ForbidEmptyCombinationOnce()
+{
+  SafeCombinationValidator *scValidator = dynamic_cast<SafeCombinationValidator *>(m_textCtrl->GetValidator());
+  if (scValidator != nullptr)
+    scValidator->ForbidEmptyCombinationOnce();
+}
+
 /**
  * Changes the textual representation of the password in the text entry field
  * between asterisks and normal character representation.
@@ -196,4 +204,4 @@ void SafeCombinationCtrl::DisableAndClear(bool disableClear)
 {
   m_textCtrl->Enable(!disableClear);
   m_textCtrl->Clear();
-}
\ No newline at end of file
+}
