@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -21,6 +21,8 @@
 #include <wx/msw/msvcrt.h>
 #endif
 
+#include <wx/filename.h>
+
 #include "os/file.h"
 
 ////@begin includes
@@ -59,6 +61,7 @@ BEGIN_EVENT_TABLE( SafeCombinationPromptDlg, wxDialog )
   EVT_BUTTON( wxID_OK,     SafeCombinationPromptDlg::OnOkClick           )
   EVT_BUTTON( wxID_CANCEL, SafeCombinationPromptDlg::OnCancelClick       )
   EVT_BUTTON( wxID_EXIT,   SafeCombinationPromptDlg::OnExitClick         )
+  EVT_CHECKBOX(ID_READONLY,SafeCombinationPromptDlg::OnReadonlyClick     )
 
 END_EVENT_TABLE()
 
@@ -66,23 +69,15 @@ END_EVENT_TABLE()
  * SafeCombinationPromptDlg constructors
  */
 
-SafeCombinationPromptDlg::SafeCombinationPromptDlg(wxWindow* parent, PWScore &core,
-                                               const wxString &fname, wxWindowID id,
+SafeCombinationPromptDlg::SafeCombinationPromptDlg(wxWindow *parent, PWScore &core,
+                                               const wxString &fname, const bool allowExit,
+                                               wxWindowID id,
                                                const wxString& caption,
                                                const wxPoint& pos,
                                                const wxSize& size, long style)
-: m_core(core), m_filename(fname), m_tries(0)
-{
-  Init();
-  Create(parent, id, caption, pos, size, style);
-}
-
-/*!
- * SafeCombinationPromptDlg creator
- */
-
-bool SafeCombinationPromptDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
+: m_core(core), m_filename(fname), m_tries(0), m_allowExit(allowExit)
 {
+  wxASSERT(!parent || parent->IsTopLevel());
 ////@begin SafeCombinationPromptDlg creation
   SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
   wxDialog::Create( parent, id, caption, pos, size, style );
@@ -101,7 +96,17 @@ bool SafeCombinationPromptDlg::Create( wxWindow* parent, wxWindowID id, const wx
   m_pollingTimer = new wxTimer(this, POLLING_TIMER_ID);
   m_pollingTimer->Start(YubiMixin::POLLING_INTERVAL);
 #endif
-  return true;
+}
+
+
+SafeCombinationPromptDlg* SafeCombinationPromptDlg::Create(wxWindow *parent, PWScore &core,
+                                               const wxString &fname, const bool allowExit,
+                                               wxWindowID id,
+                                               const wxString& caption,
+                                               const wxPoint& pos,
+                                               const wxSize& size, long style)
+{
+  return new SafeCombinationPromptDlg(parent, core, fname, allowExit, id, caption, pos, size, style);
 }
 
 /*!
@@ -117,22 +122,6 @@ SafeCombinationPromptDlg::~SafeCombinationPromptDlg()
 #endif
 }
 
-/*!
- * Member initialisation
- */
-
-void SafeCombinationPromptDlg::Init()
-{
-////@begin SafeCombinationPromptDlg member initialisation
-  m_scctrl = nullptr;
-#ifndef NO_YUBI
-  m_YubiBtn = nullptr;
-  m_yubiStatusCtrl = nullptr;
-#endif
-
-////@end SafeCombinationPromptDlg member initialisation
-}
-
 /*!
  * Control creation for SafeCombinationPromptDlg
  */
@@ -172,14 +161,29 @@ void SafeCombinationPromptDlg::CreateControls()
   m_scctrl = new SafeCombinationCtrl( itemDialog1, ID_PASSWORD, &m_password, wxDefaultPosition, wxDefaultSize );
   m_scctrl->SetFocus();
   flexGridSizer->Add(m_scctrl, 1, wxALL|wxEXPAND, 5);
-
+  
   flexGridSizer->AddStretchSpacer(0);
+
+  auto *itemBoxSizer14 = new wxBoxSizer(wxHORIZONTAL);
+
+  auto itemCheckBox15 = new wxCheckBox(this, ID_READONLY, _("Open as read-only"), wxDefaultPosition, wxDefaultSize, 0 );
+  itemCheckBox15->SetValue(false);
+  m_readOnly = false;
+
   auto showCombinationCheckBox = new wxCheckBox(this, wxID_ANY, _("Show Combination"), wxDefaultPosition, wxDefaultSize, 0 );
   showCombinationCheckBox->SetValue(false);
   showCombinationCheckBox->Bind(wxEVT_CHECKBOX, [&](wxCommandEvent& event) {m_scctrl->SecureTextfield(!event.IsChecked());});
-  flexGridSizer->Add(showCombinationCheckBox, 1, wxALL|wxEXPAND, 5);
 
+  itemBoxSizer14->Add(itemCheckBox15, 1, wxALIGN_LEFT|wxALL, 5);
+  itemBoxSizer14->AddSpacer(1);
+  itemBoxSizer14->Add(showCombinationCheckBox, 1, wxALIGN_LEFT|wxALL, 5);
+
+  flexGridSizer->Add(itemBoxSizer14, 0, wxEXPAND|wxALL, 5);
+  
   itemBoxSizer5->Add(flexGridSizer, 0, wxEXPAND|wxALL, 5);
+  
+  itemCheckBox15->SetValidator( wxGenericValidator(& m_readOnly) );
+  UpdateReadOnlyCheckbox(itemCheckBox15);
 
   auto *itemBoxSizer11 = new wxBoxSizer(wxHORIZONTAL);
   itemBoxSizer5->Add(itemBoxSizer11, 0, wxEXPAND|wxALL, 5);
@@ -222,6 +226,8 @@ void SafeCombinationPromptDlg::CreateControls()
   );
   okButton->SetDefault();
 
+  if(! m_allowExit)
+    FindWindow(wxID_EXIT)->Disable();
   // Set validators
   itemStaticText7->SetValidator( wxGenericValidator(& m_filename) );
 ////@end SafeCombinationPromptDlg content construction
@@ -295,7 +301,7 @@ void SafeCombinationPromptDlg::ProcessPhrase()
     txt->SetFocus();
     return;
   }
-  // m_core.SetReadOnly(m_readOnly);
+  m_core.SetReadOnly(m_readOnly);
   m_core.SetCurFile(tostringx(m_filename));
   EndModal(wxID_OK);
 }
@@ -347,6 +353,33 @@ void SafeCombinationPromptDlg::OnExitClick(wxCommandEvent& WXUNUSED(evt))
 ////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL in SafeCombinationPromptDlg.
 }
 
+/*!
+ * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_READONLY
+ */
+
+void SafeCombinationPromptDlg::OnReadonlyClick( wxCommandEvent& event )
+{
+  m_readOnly = event.IsChecked();
+}
+
+void SafeCombinationPromptDlg::UpdateReadOnlyCheckbox(wxCheckBox *checkBox)
+{
+  wxFileName fn(m_filename);
+
+  wxASSERT_MSG(checkBox, wxT("checkbox NULL"));
+  if (m_core.IsDbOpen()) {
+    m_readOnly = m_core.IsReadOnly();
+    checkBox->SetValue(m_readOnly);
+    checkBox->Enable(false);
+  } else if ( fn.FileExists() ) { // Do nothing if the file doesn't exist
+    bool writeable = fn.IsFileWritable();
+    bool defaultRO = PWSprefs::GetInstance()->GetPref(PWSprefs::DefaultOpenRO);
+    m_readOnly = (writeable? defaultRO : true);
+    checkBox->SetValue(m_readOnly);
+    checkBox->Enable(writeable);
+  }
+}
+
 #ifndef NO_YUBI
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_YUBIBTN
