@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -26,6 +26,7 @@
 #include <wx/statline.h>
 #include <wx/collpane.h>
 
+#include "core/core.h"
 #include "core/PWScore.h"
 
 #include "AdvancedSelectionDlg.h"
@@ -47,10 +48,11 @@ struct SyncData {
   SelectionCriteria selCriteria;
   wxFileName        otherDB;
   StringX           combination;
-  PWScore*          core;
+  PWScore*          core = nullptr;
   size_t            numUpdated;
-  CReport           syncReport;
+  std::unique_ptr<CReport> syncReport = std::unique_ptr<CReport>(new CReport());
   bool              showReport;
+  MultiCommands*    syncCmds = nullptr;
 };
 
 /*!
@@ -109,7 +111,7 @@ class DbSelectionPage : public SyncWizardPage
   DbSelectionPanel* m_panel;
 
 public:
-  DbSelectionPage(wxWizard* parent, SyncData* data);
+  DbSelectionPage(wxWizard* parent, SyncData* data, const wxString filename = "");
 
   virtual bool OnPageLeave(PageDirection dir);
   virtual void SaveData(SyncData* data);
@@ -201,7 +203,6 @@ class SyncStatusPage: public SyncWizardPage
   void Synchronize(PWScore* currentCore, const PWScore* otherCore);
   void ReportAdvancedOptions(CReport* rpt, const wxString& operation);
   void OnSyncStartEvent(wxCommandEvent& evt);
-
 public:
   SyncStatusPage(wxWizard* parent, SyncData* data);
 
@@ -214,9 +215,11 @@ public:
 //
 BEGIN_EVENT_TABLE(SyncWizard, wxWizard)
   EVT_WIZARD_PAGE_CHANGING(wxID_ANY, SyncWizard::OnWizardPageChanging)
+  EVT_CLOSE( SyncWizard::OnClose )
+  EVT_WIZARD_CANCEL(wxID_ANY, SyncWizard::OnWizardCancel)
 END_EVENT_TABLE()
 
-SyncWizard::SyncWizard(wxWindow* parent, PWScore* core):
+SyncWizard::SyncWizard(wxWindow* parent, PWScore* core, const wxString filename):
                 wxWizard(parent, wxID_ANY, _("Synchronize another database with currently open database")),
                 m_page1(nullptr), m_syncData(new SyncData)
 {
@@ -237,7 +240,7 @@ SyncWizard::SyncWizard(wxWindow* parent, PWScore* core):
 
   m_page1 = new SyncStartPage(this, m_syncData);
 
-  auto *page2 = new DbSelectionPage(this, m_syncData);
+  auto *page2 = new DbSelectionPage(this, m_syncData, filename);
   auto *page3 = new SyncFieldSelectionPage(this, m_syncData);
   auto *page4 = new SyncOptionsSummaryPage(this, m_syncData);
   auto *page5 = new SyncStatusPage(this, m_syncData);
@@ -285,7 +288,59 @@ bool SyncWizard::ShowReport() const {
 }
 
 CReport* SyncWizard::GetReport() const {
-  return &m_syncData->syncReport;
+  return m_syncData->syncReport.get();
+}
+
+MultiCommands* SyncWizard::GetSyncCommands() const {
+  return m_syncData->syncCmds;
+}
+
+bool SyncWizard::QueryCancel(bool showDialog) {
+  if (m_syncData->numUpdated > 0) {
+    if (showDialog) {
+      auto res = wxMessageDialog(
+        nullptr,
+        _("One or more items have been changed. Are you sure you wish to cancel?"), wxEmptyString,
+        wxYES_NO | wxNO_DEFAULT | wxICON_EXCLAMATION
+      ).ShowModal();
+      if (res == wxID_YES) {
+        return true;
+      }
+    }
+    return false;
+  }
+  return true;
+}
+
+void SyncWizard::ResetSyncData() {
+  // reset info about changes
+  m_syncData->numUpdated = 0;
+  m_syncData->showReport = false;
+  m_syncData->syncReport = std::unique_ptr<CReport>(new CReport());
+  if (m_syncData->syncCmds) {
+    delete m_syncData->syncCmds;
+  }
+  m_syncData->syncCmds = nullptr;
+}
+
+void SyncWizard::OnClose(wxCloseEvent &event) {
+  if (event.CanVeto()) {
+    // when trying to closing app/db, don't ask questions when data changed
+    if (!QueryCancel(!IsCloseInProgress())) {
+      event.Veto();
+      return;
+    }
+  }
+  EndDialog(wxID_CANCEL); // cancel directly (if we skip event, OnCancel will be called and ask one more time)
+}
+
+void SyncWizard::OnWizardCancel(wxWizardEvent& event) {
+  if (QueryCancel(true)) {
+    event.Skip();
+  }
+  else {
+    event.Veto();
+  }
 }
 
 ////////////////////////////////////////////
@@ -375,14 +430,14 @@ SyncStartPage::SyncStartPage(wxWizard* parent, SyncData* data) : SyncWizardPage(
 /////////////////////////////////////////
 // DbSelectionPage implementation
 //
-DbSelectionPage::DbSelectionPage(wxWizard* parent, SyncData* data):
+DbSelectionPage::DbSelectionPage(wxWizard* parent, SyncData* data, const wxString filename):
                              SyncWizardPage(parent, data, _("Select another database"))
 {
   const wxString filePrompt(wxString(_("Choose Database to Synchronize with \"")) << towxstring(data->core->GetCurFile()) << wxT("\""));
   const wxString filePickerCtrlTitle(_("Please Choose a Database to Synchronize with current database"));
 
   wxBoxSizer* sizer = m_pageSizer;
-  m_panel = new DbSelectionPanel(this, filePrompt, filePickerCtrlTitle, false, data->core, 5);
+  m_panel = new DbSelectionPanel(this, filePrompt, filePickerCtrlTitle, false, data->core, 5, wxID_OK, filename);
   sizer->Add(m_panel, wxSizerFlags().Expand().Proportion(1));
   SetSizerAndFit(sizer);
 }
@@ -634,9 +689,9 @@ Foreach entry in otherCore
 */
 void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
 {
-  CReport& rpt = m_syncData->syncReport;
+  CReport& rpt = *m_syncData->syncReport;
 
-  rpt.StartReport(_("Synchronize").c_str(), currentCore->GetCurFile().c_str());
+  rpt.StartReport(IDSC_RPTSYNCH, currentCore->GetCurFile().c_str());
   wxString line = wxString::Format(_("Synchronizing from database: %ls\n"), otherCore->GetCurFile().c_str());
   rpt.WriteLine(line.c_str());
 
@@ -741,7 +796,7 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
   Command *pcmd2 = UpdateGUICommand::Create(currentCore, UpdateGUICommand::WN_REDO,
                                             UpdateGUICommand::GUI_REDO_MERGESYNC);
   pmulticmds->Add(pcmd2);
-  currentCore->Execute(pmulticmds);
+  m_syncData->syncCmds = pmulticmds; // will execute later
 
   /* tell the user we're done & provide short merge report */
   const wxString cs_entries = (numUpdated == 1 ? _("entry") : _("entries"));
@@ -754,23 +809,5 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
 
 void SyncStatusPage::ReportAdvancedOptions(CReport* rpt, const wxString& operation)
 {
-  wxString line = m_syncData->selCriteria.GetGroupSelectionDescription();
-  line << _(" were ") << operation << _(" with corresponding entries from \"")
-              << m_syncData->otherDB.GetFullPath() << wxT('"');
-  rpt->WriteLine(line.c_str());
-
-  wxArrayString fieldsSelected, fieldsNotSelected;
-  const bool allSelected = m_syncData->selCriteria.GetFieldSelection(fieldsSelected, fieldsNotSelected);
-  if (allSelected) {
-    line.Printf(_("All fields in matching entries were %ls"), operation);
-    rpt->WriteLine(line.c_str());
-  }
-  else {
-    line.Printf(_("The following fields were %ls"), operation);
-    rpt->WriteLine(line.c_str());
-    for( size_t idx = 0; idx < fieldsSelected.Count(); ++idx) {
-      line.Printf(wxT("\t* %ls"), fieldsSelected[idx]);
-      rpt->WriteLine(line.c_str());
-    }
-  }
+  m_syncData->selCriteria.ReportAdvancedOptions(rpt, operation, m_syncData->otherDB.GetFullPath().c_str());
 }
