@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -31,9 +31,12 @@ class SyncWizard : public wxWizard
 {
   wxWizardPageSimple* m_page1;
   SyncData* m_syncData;
-
+  bool QueryCancel(bool showDialog);
+  void OnWizardCancel(wxWizardEvent& event);
+  void OnClose(wxCloseEvent &event);
+  void ResetSyncData();
 public:
-  SyncWizard(wxWindow* parent, PWScore* core);
+  SyncWizard(wxWindow* parent, PWScore* core, const wxString filename = "");
   ~SyncWizard();
 
   //users must pass the return value of this function to RunWizard
@@ -42,7 +45,8 @@ public:
 
   size_t GetNumUpdated() const;
   bool   ShowReport() const;
-  CReport*   GetReport() const;
+  CReport* GetReport() const;
+  MultiCommands* GetSyncCommands() const;
   void OnWizardPageChanging(wxWizardEvent& evt);
 
   DECLARE_EVENT_TABLE()
