@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -135,6 +135,7 @@ wxMenu* SystemTray::CreatePopupMenu()
   if (m_status != TrayStatus::CLOSED) {
     menu->AppendSeparator();
     menu->Append(wxID_CLOSE, _("&Close"))->SetBitmap(wxBitmap(close_xpm));
+    menu->Enable(wxID_CLOSE, m_frame->CanCloseDialogs());
     menu->AppendSubMenu(GetRecentHistory(), _("&Recent Entries History"));
   }
 
@@ -146,11 +147,19 @@ wxMenu* SystemTray::CreatePopupMenu()
   menu->Append(wxID_ABOUT,         _("&About Password Safe..."))->SetBitmap(wxBitmap(about_xpm));
   menu->AppendSeparator();
   menu->Append(wxID_EXIT, _("&Exit"))->SetBitmap(wxBitmap(exit_xpm));
-
+  menu->Enable(wxID_EXIT, m_frame->CanCloseDialogs());
   //let the user iconize even if its already iconized
   if (!m_frame->IsShown())
     menu->Enable(wxID_ICONIZE_FRAME, false);
 
+  // whe there are active modal dialogs, we need to reparent menu, so it could process context menu events
+  // "fix" for https://github.com/wxWidgets/wxWidgets/blob/v3.0.5/src/gtk/menu.cpp#L49
+  wxTopLevelWindow *parent = m_frame->GetTopWindow();
+  if (parent) {
+    menu->SetEventHandler(this);
+    parent->PopupMenu(menu, wxPoint(-1, -1));
+    return nullptr;
+  }
   return menu;
 }
 
@@ -230,51 +239,53 @@ void SystemTray::OnSysTrayMenuItem(wxCommandEvent& evt)
     else {
       wxCommandEvent cmd(evt.GetEventType(), GetFrameCommandId(opn));
       cmd.SetExtraLong(id);
-#if wxCHECK_VERSION(2,9,0)
       m_frame->GetEventHandler()->ProcessEvent(cmd);
-#else
-      m_frame->ProcessEvent(cmd);
-#endif
     }
   }
   else {
     switch(id) {
-
-      case ID_SYSTRAY_RESTORE:
-        m_frame->UnlockSafe(true, false); // true => restore UI
-        break;
-
-      case ID_SYSTRAY_LOCK:
-        m_frame->HideUI(true);
-        break;
-
-      case ID_SYSTRAY_UNLOCK:
-        m_frame->UnlockSafe(false, false); // false => don't restore UI
-        break;
-
-      case ID_SYSTRAY_CLEAR_RUE:
-        m_frame->ClearRUEList();
-        break;
-
       case wxID_EXIT:
       case ID_CLEARCLIPBOARD:
       case wxID_ABOUT:
       case wxID_CLOSE:
         m_frame->GetEventHandler()->ProcessEvent(evt);
         break;
-
-      case wxID_ICONIZE_FRAME:
-        m_frame->Iconize();
-        break;
-
       default:
+        wxTheApp->CallAfter([this, id](){ ProcessSysTrayMenuItem(id); });
         break;
     }
   }
 }
 
+void SystemTray::ProcessSysTrayMenuItem(int itemId)
+{
+  switch(itemId) {
+    case ID_SYSTRAY_RESTORE:
+      m_frame->UnlockSafe(true, false); // true => restore UI
+      break;
+
+    case ID_SYSTRAY_LOCK:
+      m_frame->HideUI(true);
+      break;
+
+    case ID_SYSTRAY_UNLOCK:
+      m_frame->UnlockSafe(false, false); // false => don't restore UI
+      break;
+
+    case ID_SYSTRAY_CLEAR_RUE:
+      m_frame->ClearRUEList();
+      break;
+
+    case wxID_ICONIZE_FRAME:
+      m_frame->Iconize();
+      break;
+
+    default:
+      break;
+  }
+}
+
 void SystemTray::OnTaskBarLeftDoubleClick(wxTaskBarIconEvent& WXUNUSED(evt))
 {
-  EventHandlerDisabler ehd(this);
-  m_frame->UnlockSafe(true, false); //true => restore UI
+  wxTheApp->CallAfter([this](){ ProcessSysTrayMenuItem(ID_SYSTRAY_RESTORE); });
 }
