@@ -125,6 +125,7 @@ BEGIN_EVENT_TABLE( TreeCtrl, wxTreeCtrl )
   EVT_TREE_ITEM_GETTOOLTIP( ID_TREECTRL, TreeCtrl::OnGetToolTip )
   EVT_MENU( ID_ADDGROUP, TreeCtrl::OnAddGroup )
   EVT_MENU( ID_RENAME, TreeCtrl::OnRenameGroup )
+  EVT_TREE_BEGIN_LABEL_EDIT( ID_TREECTRL, TreeCtrl::OnStartLabelEdit )
   EVT_TREE_END_LABEL_EDIT( ID_TREECTRL, TreeCtrl::OnEndLabelEdit )
   EVT_TREE_END_LABEL_EDIT( ID_TREECTRL_1, TreeCtrl::OnEndLabelEdit )
   EVT_TREE_KEY_DOWN( ID_TREECTRL, TreeCtrl::OnKeyDown )
@@ -1012,6 +1013,21 @@ void TreeCtrl::OnRenameGroup(wxCommandEvent& WXUNUSED(evt))
   }
 }
 
+void TreeCtrl::OnStartLabelEdit( wxTreeEvent& evt )
+{
+  wxTreeItemId item = evt.GetItem();
+  if(IsReadOnly() || !item.IsOk() || !ItemIsGroup(item)) {
+    // In case of read only, item not ok or item is no group editing in the tree looks not promissing
+    evt.Veto();
+    return;
+  }
+  // If old path not yet set, fill out now before editing
+  auto *data = dynamic_cast<PWTreeItemData *>(GetItemData(item));
+  if (!data) {
+    SetItemData(item, new PWTreeItemData(GetItemGroup(item)));
+  }
+}
+
 void TreeCtrl::OnEndLabelEdit( wxTreeEvent& evt )
 {
   const wxString &label = (evt.IsEditCancelled() ? GetItemText(evt.GetItem()) : evt.GetLabel());
@@ -1070,6 +1086,10 @@ void TreeCtrl::OnEndLabelEdit( wxTreeEvent& evt )
           // An existing group being renamed
           FinishRenamingGroup(evt, groupItem, data->GetOldPath());
         }
+        else {
+          // Oops, rename not possible, as starting group name unknown
+          wxFAIL_MSG(wxString::Format(wxT("ID_TREECTRL_1 no old path known")));
+        }
       }
       break;
     }
