@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -20,17 +20,20 @@
 #include <wx/msw/msvcrt.h>
 #endif
 
+#include <wx/msgdlg.h>
+
 #include "core/Report.h"
 
 #include "Clipboard.h"
 #include "ViewReportDlg.h"
 #include "wxUtilities.h"
 
-ViewReportDlg::ViewReportDlg(wxWindow* parent, CReport* pRpt) :
+ViewReportDlg::ViewReportDlg(wxWindow *parent, CReport* pRpt, bool fromFile) :
                 wxDialog(parent, wxID_ANY, _("View Report"), wxDefaultPosition, wxDefaultSize,
                       wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),  m_pRpt(pRpt)
 {
   wxASSERT(pRpt);
+  wxASSERT(!parent || parent->IsTopLevel());
 
   wxBoxSizer* dlgSizer = new wxBoxSizer(wxVERTICAL);
 
@@ -42,7 +45,12 @@ ViewReportDlg::ViewReportDlg(wxWindow* parent, CReport* pRpt) :
 
   wxASSERT_MSG(bs, wxT("Could not create an empty wxStdDlgButtonSizer"));
 
-  bs->Add(new wxButton(this, wxID_SAVE, _("&Save to Disk")));
+  if(fromFile) {
+    bs->Add(new wxButton(this, wxID_APPLY, _("&Remove from Disk")));
+  }
+  else {
+    bs->Add(new wxButton(this, wxID_SAVE, _("&Save to Disk")));
+  }
   bs->AddSpacer(ColSeparation);
   bs->Add(new wxButton(this, wxID_COPY, _("&Copy to Clipboard")));
   bs->AddSpacer(ColSeparation);
@@ -53,6 +61,7 @@ ViewReportDlg::ViewReportDlg(wxWindow* parent, CReport* pRpt) :
   bs->Realize();
 
   Connect(wxID_SAVE, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ViewReportDlg::OnSave) );
+  Connect(wxID_APPLY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ViewReportDlg::OnRemove) );
   Connect(wxID_COPY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ViewReportDlg::OnCopy) );
   Connect(wxID_CLOSE, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ViewReportDlg::OnClose) );
 
@@ -61,8 +70,9 @@ ViewReportDlg::ViewReportDlg(wxWindow* parent, CReport* pRpt) :
   SetSizerAndFit(dlgSizer);
 }
 
-ViewReportDlg::~ViewReportDlg()
+ViewReportDlg* ViewReportDlg::Create(wxWindow *parent, CReport* pRpt, bool fromFile)
 {
+  return new ViewReportDlg(parent, pRpt, fromFile);
 }
 
 void ViewReportDlg::OnSave(wxCommandEvent& evt)
@@ -71,6 +81,16 @@ void ViewReportDlg::OnSave(wxCommandEvent& evt)
   m_pRpt->SaveToDisk();
 }
 
+void ViewReportDlg::OnRemove(wxCommandEvent& evt)
+{
+  UNREFERENCED_PARAMETER(evt);
+  wxString fileName(m_pRpt->GetFileName());
+  wxMessageDialog dlg(this, fileName, _("Delete Report?"), wxYES_NO);
+  if(dlg.ShowModal() == wxID_YES) {
+    m_pRpt->PurgeFromDisk();
+  }
+}
+
 void ViewReportDlg::OnClose(wxCommandEvent& evt)
 {
   UNREFERENCED_PARAMETER(evt);
