@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -22,10 +22,12 @@ class ViewReportDlg : public wxDialog {
   CReport* m_pRpt;
   
 public:
-  ViewReportDlg(wxWindow* pParent, CReport* pRpt);
-  ~ViewReportDlg();
-
+  static ViewReportDlg* Create(wxWindow *parent, CReport* pRpt, bool fromFile = false);
+  ~ViewReportDlg() = default;
+protected:
+  ViewReportDlg(wxWindow *parent, CReport* pRpt, bool fromFile);
   void OnSave(wxCommandEvent& event);
+  void OnRemove(wxCommandEvent& event);
   void OnClose(wxCommandEvent& event);
   void OnCopy(wxCommandEvent& event);
 };
