@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -176,9 +176,6 @@ inline const wxChar* ToStr(bool b) {
   return b? wxT("True"): wxT("False");
 }
 
-void HideWindowRecursively(wxTopLevelWindow* win, wxWindowList& hiddenWindows);
-void ShowWindowRecursively(wxWindowList& hiddenWindows);
-
 // Workaround for wxTE_PASSWORD being immutable:
 void ShowHideText(wxTextCtrl *&txtCtrl, const wxString &text,
                   wxSizer *sizer, bool show);
@@ -401,4 +398,32 @@ private:
   wxBitmap m_Bitmap;          // The possibly scaled bitmap representation of the image.
 };
 
+bool IsCloseInProgress();
+
+template <class Dlg, typename... Args> int ShowModalAndGetResult(Args&&... args) {
+  Dlg* ptr = Dlg::Create(std::forward<Args>(args)...);
+  int result = ptr->ShowModal();
+  ptr->Destroy();
+  return result;
+}
+
+template<class T> class DestroyWrapper {
+public:
+  template<typename... Args>
+  DestroyWrapper(Args&&... args) : m_ptr(T::Create(std::forward<Args>(args)...)) {
+  }
+
+  ~DestroyWrapper() {
+    if (m_ptr) {
+      m_ptr->Destroy();
+    }
+  }
+
+  T* Get() {
+    return m_ptr;
+  }
+private:
+  T *m_ptr;
+};
+
 #endif // _WXUTILITIES_H_
